@@ -8,6 +8,7 @@ Created on Sun Dec  4 18:14:29 2016
 import numpy as np
 import scipy.sparse as spsp
 import scipy.sparse.linalg as splinalg
+from scipy.optimize import root
 
 import simfempy.tools.analyticalfunction
 import simfempy.tools.timer
@@ -43,7 +44,7 @@ class Application(object):
         self.mode = kwargs.pop('mode', 'linear')
         self.verbose = kwargs.pop('verbose', 0)
         self.linearsolver = kwargs.pop('linearsolver', 'umf')
-        self.timer = simfempy.tools.timer.Timer(verbose=0)
+        self.timer = simfempy.tools.timer.Timer(verbose=self.verbose)
         if 'problemdata' in kwargs:
             self.problemdata = kwargs.get('problemdata')
             self.ncomp = self.problemdata.ncomp
@@ -72,8 +73,8 @@ class Application(object):
             self.generatePoblemDataForAnalyticalSolution()
             self._generatePDforES = False
     def solve(self, dirname="Run"): return self.static(dirname=dirname, mode=self.mode)
-    def setParameter(self, paramname, param):
-        assert 0
+    # def setParameter(self, paramname, param):
+    #     assert 0
     def dirichletfct(self):
         if self.ncomp > 1:
             # def _solexactdir(x, y, z):
@@ -144,23 +145,35 @@ class Application(object):
         self.timer.add('rhs')
         if mode == 'linear':
             try:
-                u, niter = self.linearSolver(self.A, self.b, u, solver=self.linearsolver)
+                u, niterlin = self.linearSolver(self.A, self.b, u, linearsolver=self.linearsolver)
             except Warning:
                 raise ValueError(f"matrix is singular {self.A.shape=} {self.A.diagonal()=}")
             self.timer.add('solve')
+            iter={'lin':niterlin}
         else:
             if 'sdata' in kwargs:
                 sdata = kwargs.pop('sdata')
             else:
                 maxiter = kwargs.pop('maxiter', 100)
                 sdata = simfempy.solvers.newtondata.StoppingData(maxiter=maxiter)
-            info = simfempy.solvers.newton.newton(u, f=self.computeDefect, computedx=self.computeDx, verbose=True, sdata=sdata)
-            # print(f"{info=}")
-            u,niter = info
-            if niter==sdata.maxiter: raise ValueError(f"*** Attained maxiter {sdata.maxiter=}")
+            sdata.addname = self.linearsolver
+            if mode == 'newton':
+                info = simfempy.solvers.newton.newton(u, f=self.computeDefect, computedx=self.computeDx, verbose=True, sdata=sdata)
+                # print(f"{info=}")
+                u,niter,niterlin = info
+                iter={'lin':niterlin, 'nlin':niter}
+                if niter==sdata.maxiter: raise ValueError(f"*** Attained maxiter {sdata.maxiter=}")
+            elif mode == 'newtonkrylov':
+                counter = simfempy.tools.iterationcounter.IterationCounterWithRes(name=mode, disp=1, callback_type='x,Fx')
+                n = u.shape[0]
+                prec = splinalg.LinearOperator(shape=(n,n), matvec=lambda x: self.computePrec(x))
+                root(fun=self.computeDefect, x0=u, args=(), method='krylov', tol=sdata.rtol, callback=counter, 
+                options={'inner_M': prec})
+            else:
+                raise ValueError(f"unknwon {mode=}")
         pp = self.postProcess(u)
         self.timer.add('postp')
-        result.setData(pp, timer=self.timer, iter={'lin':niter})
+        result.setData(pp, timer=self.timer, iter=iter)
         return result
     def computeDefect(self, u):
         return self.computeForm(u)-self.b
@@ -257,18 +270,18 @@ class Application(object):
             if callback: callback(self.time, u)
         return result
 
-    def linearSolver(self, A, b, u=None, solver = None, verbose=0):
+    def linearSolver(self, A, b, u=None, linearsolver = None, verbose=0, disp=0):
         if spsp.issparse(A):
             if len(b.shape)!=1 or len(A.shape)!=2 or b.shape[0] != A.shape[0]:
                 raise ValueError(f"{A.shape=} {b.shape=}")
-        if solver is None: solver = self.linearsolver
+        if linearsolver is None: solver = self.linearsolver
         if not hasattr(self, 'info'): self.info={}
-        if solver not in self.linearsolvers: solver = "umf"
-        if solver == 'umf':
+        if linearsolver not in self.linearsolvers: solver = "umf"
+        if linearsolver == 'umf':
             return splinalg.spsolve(A, b), 1
             return splinalg.spsolve(A, b, permc_spec='COLAMD'), 1
-        elif solver in ['gmres','lgmres','bicgstab','cg']:
-            if solver == 'cg':
+        elif linearsolver in ['gmres','lgmres','bicgstab','cg','gcrotmk']:
+            if linearsolver == 'cg':
                 def gaussSeidel(A):
                     dd = A.diagonal()
                     D = spsp.dia_matrix(A.shape)
@@ -280,14 +293,14 @@ class Application(object):
             else:
                 # defaults: drop_tol=0.0001, fill_factor=10
                 M2 = splinalg.spilu(A.tocsc(), drop_tol=0.1, fill_factor=3)
-            M_x = lambda x: M2.solve(x)
-            M = splinalg.LinearOperator(A.shape, M_x)
-            counter = simfempy.tools.iterationcounter.IterationCounter(name=solver, verbose=verbose)
+            M = splinalg.LinearOperator(A.shape, lambda x: M2.solve(x))
+            counter = simfempy.tools.iterationcounter.IterationCounter(name=linearsolver, disp=disp)
             args=""
-            cmd = "u = splinalg.{}(A, b, M=M, tol=1e-14, callback=counter {})".format(solver,args)
-            exec(cmd)
+            cmd = "splinalg.{}(A, b, M=M, tol=1e-14, callback=counter {})".format(linearsolver,args)
+            u, info = eval(cmd)
+            # print(f"{u=}")
             return u, counter.niter
-        elif solver == 'pyamg':
+        elif linearsolver == 'pyamg':
             ml = self.build_pyamg(A)
             maxiter = 100
             u, res = self.solve_pyamg(ml, b, u, maxiter)
@@ -295,7 +308,7 @@ class Application(object):
             if(verbose): print('niter ({}) {:4d} ({:7.1e})'.format(solver, len(res),res[-1]/res[0]))
             return u, len(res)
         else:
-            raise NotImplementedError("unknown solve '{}'".format(solver))
+            raise NotImplementedError("unknown solve '{}'".format(linearsolver))
     def pyamg_solver_args(self, maxiter):
         return {'cycle': 'V', 'maxiter': maxiter, 'tol': 1e-12, 'accel': 'gmres'}
     def solve_pyamg(self, ml, b, u, maxiter):
@@ -306,42 +319,6 @@ class Application(object):
     def build_pyamg(self,A):
         import pyamg
         return pyamg.smoothed_aggregation_solver(A)
-        B = np.ones((A.shape[0], 1))
-
-
-    # def solveNonlinearProblem(self, u=None, sdata=None, method="newton", checkmaxiter=True):
-    #     if not hasattr(self,'mesh'): raise ValueError("*** no mesh given ***")
-    #     self.timer.add('init')
-    #     A = self.matrix()
-    #     self.timer.add('matrix')
-    #     self.b,u = self.computeRhs(u)
-    #     self.du = np.empty_like(u)
-    #     self.timer.add('rhs')
-    #     if method == 'newton':
-    #         from . import newton
-    #         u, nit = newton.newton(x0=u, f=self.residualNewton, computedx=self.solveForNewton, sdata=sdata, verbose=True)
-    #     elif method in ['broyden2','krylov', 'df-sane', 'anderson']:
-    #         sol = optimize.root(self.residualNewton, u, method=method)
-    #         u, nit = sol.x, sol.nit
-    #     else:
-    #         raise ValueError(f"unknown method {method}")
-    #     point_data, cell_data, info = self.postProcess(u)
-    #     self.timer.add('postp')
-    #     info['timer'] = self.timer
-    #     info['iter'] = {'lin':nit}
-    #     return point_data, cell_data, info
-
-    # def residualNewton(self, u):
-    #     # print(f"self.b={self.b.shape}")
-    #     self.A = self.matrix()
-    #     return self.A.dot(u) - self.b
-    #
-    # def solveForNewton(self, r, x):
-    #     # print(f"solveForNewton r={np.linalg.norm(r)}")
-    #     du, niter = self.linearSolver(self.A, r, x, verbose=0)
-    #     # print(f"solveForNewton du={np.linalg.norm(du)}")
-    #     return du
-
 
 
 # ------------------------------------- #
