@@ -89,7 +89,6 @@ class Heat(Application):
         return colors
     def _checkProblemData(self):
         if self.verbose: print(f"checking problem data {self.problemdata=}")
-        self.problemdata.check(self.mesh)
         if self.convection:
             convection_given = self.problemdata.params.fct_glob['convection']
             if not isinstance(convection_given, list):
@@ -152,7 +151,7 @@ class Heat(Application):
             return rhs
         return _fctrobin
     def setParameter(self, paramname, param):
-        if paramname == "dirichlet_al": self.fem.dirichlet_al = param
+        if paramname == "dirichlet_strong": self.fem.dirichlet_strong = param
         else:
             if not hasattr(self, self.paramname):
                 raise NotImplementedError("{} has no paramater '{}'".format(self, self.paramname))
@@ -170,13 +169,13 @@ class Heat(Application):
         elif self.dirichletmethod=="nitsche":
             self.fem.computeFormNitscheDiffusion(du, u, self.kheatcell, colorsdir)
         # elif self.dirichletmethod=="strong":
-        #     self.fem.vectorBoundaryEqual(u, self.b, self.bdrydata, self.dirichletmethod)
+        #     self.fem.vectorBoundaryStrongEqual(u, self.b, self.bdrydata, self.dirichletmethod)
         if self.convection:
             self.fem.computeFormConvection(du, u, self.convdata, self.convmethod)
         if coeffmass is not None:
             self.fem.massDot(du, u, coeff=coeffmass)
         if self.dirichletmethod=="strong":
-            self.fem.vectorBoundaryEqual(du, u, self.bdrydata)
+            self.fem.vectorBoundaryStrongEqual(du, u, self.bdrydata)
         if not np.allclose(du,du2):
             # f = (f"\n{du[self.bdrydata.facesdirall]}\n{du2[self.bdrydata.facesdirall]}")
             raise ValueError(f"\n{du=}\n{du2=}")
@@ -187,9 +186,9 @@ class Heat(Application):
         colorsdir = bdrycond.colorsOfType("Dirichlet")
         A = self.fem.computeMatrixDiffusion(self.kheatcell)
         if self.dirichletmethod=="new":
-            A = self.fem.matrixBoundary(A, self.bdrydata, self.dirichletmethod)
+            A = self.fem.matrixBoundaryStrong(A, self.bdrydata, self.dirichletmethod)
         elif self.dirichletmethod=="nitsche":
-            A = self.fem.computeMatrixNitscheDiffusion(diffcoff=self.kheatcell, colorsdir=colorsdir)
+            A += self.fem.computeMatrixNitscheDiffusion(diffcoff=self.kheatcell, colors=colorsdir)
         if self.verbose: print(f"{A.diagonal()=}")
         A += self.fem.computeBdryMassMatrix(colorsrobin, bdrycond.param, lumped=True)
         if self.convection:
@@ -197,7 +196,7 @@ class Heat(Application):
         if coeffmass is not None:
             A += self.fem.computeMassMatrix(coeff=coeffmass)
         if self.dirichletmethod=="strong":
-            A = self.fem.matrixBoundary(A, self.bdrydata, self.dirichletmethod)
+            A = self.fem.matrixBoundaryStrong(A, self.bdrydata, self.dirichletmethod)
         return A
     def computeRhs(self, b=None, coeffmass=None, u=None):
         if b is None:
@@ -221,7 +220,7 @@ class Heat(Application):
         if self.dirichletmethod in ['strong','new'] and not hasattr(self.bdrydata,"A_inner_dir"):
             raise ValueError("matrix() has to be called befor computeRhs()")
         if self.dirichletmethod=="new":
-            self.fem.vectorBoundary(b, bdrycond, self.bdrydata, self.dirichletmethod)
+            self.fem.vectorBoundaryStrong(b, bdrycond, self.bdrydata, self.dirichletmethod)
         elif self.dirichletmethod=="nitsche":
             fp1 = self.fem.interpolateBoundary(colorsdir, bdrycond.fct)
             self.fem.computeRhsNitscheDiffusion(b, self.kheatcell, colorsdir, fp1)
@@ -232,11 +231,11 @@ class Heat(Application):
         fp1 = self.fem.interpolateBoundary(colorsrobin, bdrycond.fct, lumped=True)
         self.fem.massDotBoundary(b, fp1, colors=colorsrobin, lumped=True, coeff=bdrycond.param)
         #Neumann
-        fp1 = self.fem.interpolateBoundary(colorsneu, bdrycond.fct, lumped=True)
+        fp1 = self.fem.interpolateBoundary(colorsneu, bdrycond.fct)
         if self.dirichletmethod == "new":
             b2 = np.zeros_like(b)
             self.fem.massDotBoundary(b2, fp1, colorsneu)
-            self.fem.vectorBoundaryZero(b2, self.bdrydata)
+            self.fem.vectorBoundaryStrongZero(b2, self.bdrydata)
             b += b2
         else:
             self.fem.massDotBoundary(b, fp1, colorsneu)
@@ -244,7 +243,7 @@ class Heat(Application):
             assert u is not None
             self.fem.massDot(b, u, coeff=coeffmass)
         if self.dirichletmethod == "strong" or self.dirichletmethod == "new":
-            self.fem.vectorBoundary(b, bdrycond, self.bdrydata, self.dirichletmethod)
+            self.fem.vectorBoundaryStrong(b, bdrycond, self.bdrydata, self.dirichletmethod)
         return b
     def postProcess(self, u):
         # TODO: virer 'error' et 'postproc'
@@ -258,7 +257,7 @@ class Heat(Application):
             data['global']['err_Flux'] = self.fem.computeErrorFluxL2(self.problemdata.solexact, u, self.kheatcell)
             data['cell']['err'] = ec
         if self.problemdata.postproc:
-            types = ["bdry_mean", "bdry_fct", "bdry_nflux", "pointvalues", "meanvalues"]
+            types = ["bdry_mean", "bdry_fct", "bdry_nflux", "pointvalues", "meanvalues", "linemeans"]
             for name, type in self.problemdata.postproc.type.items():
                 colors = self.problemdata.postproc.colors(name)
                 if type == types[0]:
@@ -275,22 +274,25 @@ class Heat(Application):
                     data['global'][name] = self.fem.computePointValues(u, colors)
                 elif type == types[4]:
                     data['global'][name] = self.fem.computeMeanValues(u, colors)
+                elif type == types[5]:
+                    data['global'][name] = self.fem.computeLineValues(u, colors)
                 else:
                     raise ValueError(f"unknown postprocess type '{type}' for key '{name}'\nknown types={types=}")
         if self.kheatcell.shape[0] != self.mesh.ncells:
             raise ValueError(f"self.kheatcell.shape[0]={self.kheatcell.shape[0]} but self.mesh.ncells={self.mesh.ncells}")
         data['cell']['k'] = self.kheatcell
         return data
-    def pyamg_solver_args(self, maxiter):
-        return {'cycle': 'W', 'maxiter': maxiter, 'tol': 1e-12, 'accel': 'bicgstab'}
+    # def pyamg_solver_args(self, maxiter):
+    #     return {'cycle': 'W', 'maxiter': maxiter, 'tol': 1e-12, 'accel': 'bicgstab'}
+    def setup_own_gs(**kwargs):
+        print(f"{kwargs=}")
     def own_gs(A, b):
         x = np.zeros_like(b)
         return x
-
     def pyamg_solver_args(self, maxiter):
         if self.convection:
-            return {'cycle': 'V', 'maxiter': maxiter, 'tol': 1e-12, 'accel': 'bicgstab'}
-        return {'cycle': 'V', 'maxiter': maxiter, 'tol': 1e-12, 'accel': 'cg'}
+            return {'cycle': 'V', 'maxiter': maxiter, 'tol': 1e-10, 'accel': 'bicgstab'}
+        return {'cycle': 'V', 'maxiter': maxiter, 'tol': 1e-10, 'accel': 'cg'}
     def build_pyamg(self, A):
         import pyamg
         # return pyamg.smoothed_aggregation_solver(A)
@@ -301,8 +303,10 @@ class Heat(Application):
             # smoother = 'gauss_seidel_nr'
             smoother = 'gauss_seidel'
             smoother = 'block_gauss_seidel'
-            # smoother = 'strength_based_schwarz'
+            smoother = 'strength_based_schwarz'
             smoother = 'schwarz'
+            # smoother = 'own_gs'
+            # smoother = 'gmres'
 
             # global setup_own_gs
             # def setup_own_gs(lvl, iterations=2, sweep='forward'):
@@ -314,7 +318,7 @@ class Heat(Application):
             # smoother = 'own_gs'
             # smooth = ('energy', {'krylov': 'fgmres'})
             smooth = ('energy', {'krylov': 'bicgstab'})
-            # improve_candidates =[ ('gauss_seidel', {'sweep': 'symmetric', 'iterations': 1}), None]
+            # improve_candidates =[ (smoother, {'sweep': 'symmetric', 'iterations': 4}), None]
             improve_candidates = None
         else:
             symmetry = 'hermitian'
@@ -322,19 +326,24 @@ class Heat(Application):
             smoother = 'gauss_seidel'
             # improve_candidates =[ ('gauss_seidel', {'sweep': 'symmetric', 'iterations': 4}), None]
             improve_candidates = None
-        strength = [('evolution', {'k': 2, 'epsilon': 10.0})]
-        presmoother = (smoother, {'sweep': 'symmetric', 'iterations': 2})
-        postsmoother = (smoother, {'sweep': 'symmetric', 'iterations': 2})
+        # strength = [('evolution', {'k': 2, 'epsilon': 10.0})]
+        strength = [('symmetric', {'theta': 0.05})]
+        psmoother = (smoother, {'sweep': 'symmetric', 'iterations': 1})
+        # psmoother = (smoother, {'maxiter': 10})
         SA_build_args = {
             'max_levels': 10,
             'max_coarse': 25,
             'coarse_solver': 'pinv2',
-            'symmetry': symmetry
+            'symmetry': symmetry,
+            'smooth': smooth,
+            'strength': strength,
+            'presmoother': psmoother,
+            'presmoother': psmoother,
+            'improve_candidates': improve_candidates,
+            'diagonal_dominance': False
         }
-        return pyamg.smoothed_aggregation_solver(A, B, smooth=smooth, strength=strength, presmoother=presmoother,
-                                                 postsmoother=postsmoother, improve_candidates=improve_candidates,
-                                                **SA_build_args)
-        # return pyamg.rootnode_solver(A, B, **SA_build_args)
+        # return pyamg.smoothed_aggregation_solver(A, B, **SA_build_args)
+        return pyamg.rootnode_solver(A, B, **SA_build_args)
 
 
 #=================================================================#
