@@ -1,3 +1,4 @@
+from matplotlib import colors
 import numpy as np
 import scipy.sparse as sparse
 import scipy.sparse.linalg as splinalg
@@ -18,12 +19,17 @@ class Stokes(Application):
             return repr
         return self.__repr__()
     def __init__(self, **kwargs):
-        self.dirichlet_nitsche = 4
+        self.dirichlet_nitsche = 10
         self.dirichletmethod = kwargs.pop('dirichletmethod', 'nitsche')
         self.problemdata = kwargs.pop('problemdata')
         self.ncomp = self.problemdata.ncomp
         self.femv = fems.cr1sys.CR1sys(self.ncomp)
         self.femp = fems.d0.D0()
+        self.hdivpenalty = kwargs.pop('hdivpenalty', 0)
+        if not 'linearsolver' in kwargs: kwargs['linearsolver'] = 'gcrotmk_0'
+        self.precond_p = kwargs.pop('precond_p', 'scale')
+        # self.precond_p = kwargs.pop('precond_p', 'schur@diag@1')
+        self.precond_v = kwargs.pop('precond_v', 'pyamg@aggregation@none@gauss_seidel')
         super().__init__(**kwargs)
     def _zeros(self):
         nv = self.mesh.dimension*self.mesh.nfaces
@@ -38,22 +44,31 @@ class Stokes(Application):
         return np.split(x, ind)
     def setMesh(self, mesh):
         super().setMesh(mesh)
+        self._checkProblemData()
         assert self.ncomp==self.mesh.dimension
         self.femv.setMesh(self.mesh)
         self.femp.setMesh(self.mesh)
         self.mucell = self.compute_cell_vector_from_params('mu', self.problemdata.params)
         # self.pmean = list(self.problemdata.bdrycond.type.values()) == len(self.problemdata.bdrycond.type)*['Dirichlet']
-        self.pmean = not ('Neumann' in self.problemdata.bdrycond.type.values())
+        self.pmean = not ('Neumann' in self.problemdata.bdrycond.type.values() or 'Pressure' in self.problemdata.bdrycond.type.values())
         if self.dirichletmethod=='strong':
             assert 'Navier' not in self.problemdata.bdrycond.type.values()
             colorsdirichlet = self.problemdata.bdrycond.colorsOfType("Dirichlet")
             colorsflux = self.problemdata.postproc.colorsOfType("bdry_nflux")
             self.bdrydata = self.femv.prepareBoundary(colorsdirichlet, colorsflux)
+    def _checkProblemData(self):
+        # TODO checkProblemData() incomplete
+        for col, fct in self.problemdata.bdrycond.fct.items():
+            type = self.problemdata.bdrycond.type[col]
+            if type == "Dirichlet":
+                if len(fct) != self.mesh.dimension: raise ValueError(f"*** {type=} {len(fct)=} {self.mesh.dimension=}")
     def defineAnalyticalSolution(self, exactsolution, random=True):
         dim = self.mesh.dimension
         # print(f"defineAnalyticalSolution: {dim=} {self.ncomp=}")
         if exactsolution=="Linear":
             exactsolution = ["Linear", "Constant"]
+        elif exactsolution=="Quadratic":
+            exactsolution = ["Quadratic", "Linear"]
         v = analyticalSolution(exactsolution[0], dim, dim, random)
         p = analyticalSolution(exactsolution[1], dim, 1, random)
         return v,p
@@ -74,7 +89,7 @@ class Stokes(Application):
         v,p = solexact
         mu = self.problemdata.params.scal_glob['mu']
         def _fctrhsv(x, y, z):
-            rhsv = np.zeros(shape=(self.ncomp, x.shape[0]))
+            rhsv = np.zeros(shape=(self.ncomp, *x.shape))
             for i in range(self.ncomp):
                 for j in range(self.ncomp):
                     rhsv[i] -= mu * v[i].dd(j, j, x, y, z)
@@ -82,7 +97,7 @@ class Stokes(Application):
             # print(f"{rhsv=}")
             return rhsv
         def _fctrhsp(x, y, z):
-            rhsp = np.zeros(x.shape[0])
+            rhsp = np.zeros(x.shape)
             for i in range(self.ncomp):
                 rhsp += v[i].d(i, x, y, z)
             return rhsp
@@ -90,23 +105,56 @@ class Stokes(Application):
     def defineNeumannAnalyticalSolution(self, problemdata, color):
         solexact = problemdata.solexact
         mu = self.problemdata.params.scal_glob['mu']
-        def _fctneumannv(x, y, z, nx, ny, nz):
+        def _fctneumannv(x, y, z, nx, ny, nz, icomp):
             v, p = solexact
-            rhsv = np.zeros(shape=(self.ncomp, x.shape[0]))
+            rhsv = np.zeros(shape=x.shape)
             normals = nx, ny, nz
+            # for i in range(self.ncomp):
+            for j in range(self.ncomp):
+                rhsv += mu  * v[icomp].d(j, x, y, z) * normals[j]
+            rhsv -= p(x, y, z) * normals[icomp]
+            return rhsv
+        return [partial(_fctneumannv, icomp=icomp) for icomp in range(self.ncomp)]
+    def defineNavierAnalyticalSolution(self, problemdata, color):
+        solexact = problemdata.solexact
+        mu = self.problemdata.params.scal_glob['mu']
+        lambdaR = self.problemdata.params.scal_glob['navier']
+        def _fctnaviervn(x, y, z, nx, ny, nz):
+            v, p = solexact
+            rhs = np.zeros(shape=x.shape)
+            normals = nx, ny, nz
+            # print(f"{x.shape=} {nx.shape=} {normals[0].shape=}")
+            for i in range(self.ncomp):
+                rhs += v[i](x, y, z) * normals[i]
+            return rhs
+        def _fctnaviertangent(x, y, z, nx, ny, nz, icomp):
+            v, p = solexact
+            rhs = np.zeros(shape=x.shape)
+            # h = np.zeros(shape=(self.ncomp, x.shape[0]))
+            normals = nx, ny, nz
+            rhs = lambdaR*v[icomp](x, y, z)
+            for j in range(self.ncomp):
+                rhs += mu*v[icomp].d(j, x, y, z) * normals[j]
+            return rhs
+        return {'vn':_fctnaviervn, 'g':[partial(_fctnaviertangent, icomp=icomp) for icomp in range(self.ncomp)]}
+    def definePressureAnalyticalSolution(self, problemdata, color):
+        solexact = problemdata.solexact
+        mu = self.problemdata.params.scal_glob['mu']
+        lambdaR = self.problemdata.params.scal_glob['navier']
+        def _fctpressure(x, y, z, nx, ny, nz):
+            v, p = solexact
+            # rhs = np.zeros(shape=x.shape)
+            normals = nx, ny, nz
+            # print(f"{x.shape=} {nx.shape=} {normals[0].shape=}")
+            rhs = 1.0*p(x,y,z)
             for i in range(self.ncomp):
                 for j in range(self.ncomp):
-                    rhsv[i] += mu  * v[i].d(j, x, y, z) * normals[j]
-                rhsv[i] -= p(x, y, z) * normals[i]
-            return rhsv
-        def _fctneumannp(x, y, z, nx, ny, nz):
+                    rhs -= mu*v[j].d(i, x, y, z) * normals[i]* normals[j]
+            return rhs
+        def _fctpressurevtang(x, y, z, nx, ny, nz, icomp):
             v, p = solexact
-            rhsp = np.zeros(shape=x.shape[0])
-            normals = nx, ny, nz
-            for i in range(self.ncomp):
-                rhsp -= v[i](x, y, z) * normals[i]
-            return rhsp
-        return _fctneumannv
+            return v[icomp](x,y,z)
+        return {'p':_fctpressure, 'v':[partial(_fctpressurevtang, icomp=icomp) for icomp in range(self.ncomp)]}
     def postProcess(self, u):
         if self.pmean: v, p, lam = self._split(u)
         else: v, p = self._split(u)
@@ -124,18 +172,22 @@ class Stokes(Application):
             err, e = self.femp.computeErrorL2(self.problemdata.solexact[1], p)
             data['global']['error_P_L2'] = err
         if self.problemdata.postproc:
-            types = ["bdry_pmean", "bdry_nflux"]
+            types = ["bdry_pmean", "bdry_vmean", "bdry_nflux"]
             for name, type in self.problemdata.postproc.type.items():
                 colors = self.problemdata.postproc.colors(name)
                 if type == types[0]:
                     data['global'][name] = self.femp.computeBdryMean(p, colors)
                 elif type == types[1]:
+                    data['global'][name] = self.femv.computeBdryMean(v, colors)
+                elif type == types[2]:
                     if self.dirichletmethod=='strong':
                         data['global'][name] = self.computeBdryNormalFluxStrong(v, p, colors)
                     else:
                         data['global'][name] = self.computeBdryNormalFluxNitsche(v, p, colors)
                 else:
                     raise ValueError(f"unknown postprocess type '{type}' for key '{name}'\nknown types={types=}")
+        if hasattr(self.problemdata.postproc, "changepostproc"):
+            self.problemdata.postproc.changepostproc(data['global'])
         return data
     def _to_single_matrix(self, Ain):
         ncells, nfaces = self.mesh.ncells, self.mesh.nfaces
@@ -170,36 +222,38 @@ class Stokes(Application):
             q = B.dot(v)+C.T.dot(lam)
             return np.hstack([w, q, C.dot(p)])
         else:
-            A, B = Ain
-            v, p = x[:ncomp*nfaces], x[ncomp*nfaces:]
-            w = A.dot(v) - B.T.dot(p)
-            q = B.dot(v)
+            try:
+                A, B = Ain
+                v, p = x[:ncomp*nfaces], x[ncomp*nfaces:]
+                w = A.dot(v) - B.T.dot(p)
+                q = B.dot(v)
+            except:
+                raise ValueError(f" {v.shape=} {p.shape=}  {A.shape=} {B.shape=}")
             return np.hstack([w, q])
     def getPrecMult(self, Ain, AP, SP):
         A, B = Ain[0], Ain[1]
         ncells, nfaces, ncomp = self.mesh.ncells, self.mesh.nfaces, self.ncomp
-        # mu = self.problemdata.params.scal_glob['mu']
-        # BP = sparse.diags(1/self.mesh.dV*mu, offsets=(0), shape=(ncells, ncells))
-        # Ainv = sparse.linalg.spilu(A)
-        # Ainv = sparse.diags(1/A.diagonal(), offsets=(0), shape=(nfaces*ncomp, nfaces*ncomp))
-        # S = (B*Ainv*B.T).diagonal()
-        # BP = splinalg.inv(B*Ainv*B.T)
-        # BP = sparse.diags(1/S, offsets=(0), shape=(ncells, ncells))
         if self.pmean: 
-            C = Ain[2]
-            BPCT = SP.solve(C.T.toarray())
+            C = Ain[2].A.ravel()
+            BPCT = SP.solve(C)
+            # BPCT = SP.prec.solve(C.T.toarray(), maxiter=1, tol=1e-10)
+            # print(f"{BPCT=}")            
             # print(f"{C.dot(BPCT)=}")
             # CP = splinalg.inv(C.dot(BPCT))
             CP = sparse.coo_matrix(1/C.dot(BPCT))
+            print(f"{CP.A=}")
         if self.pmean: 
             def pmult(x):
                 v, p, lam = x[:ncomp*nfaces], x[ncomp*nfaces:ncomp*nfaces+ncells], x[-1]*np.ones(1)
                 # return np.hstack([API.solve(v, maxiter=1, tol=1e-16), BP.dot(p), CP.dot(lam)])
                 w = AP.solve(v)
-                w = AP.solve(v)
                 q = SP.solve(p-B.dot(w))
                 mu = CP.dot(lam-C.dot(q)).ravel()
-                q -= BPCT.dot(mu).ravel()
+                # print(f"{mu.shape=} {lam.shape=} {BPCT.shape=}")
+                # q -= BPCT.dot(mu)
+                q -= BPCT*mu
+                # print(f"{BPCT.shape=} {mu=}")
+                # q -= mu*BPCT
                 h = B.T.dot(q)
                 w += AP.solve(h)
                 return np.hstack([w, q, mu])
@@ -207,44 +261,66 @@ class Stokes(Application):
             def pmult(x):
                 v, p = x[:ncomp*nfaces], x[ncomp*nfaces:ncomp*nfaces+ncells]
                 w = AP.solve(v)
+                # print(f"{np.linalg.norm(v)=} {np.linalg.norm(p)=}")
+                # print(f"{np.linalg.norm(w)=} {np.linalg.norm(p-B.dot(w))=}")
                 q = SP.solve(p-B.dot(w))
+                # print(f"{np.linalg.norm(q)=}")
                 h = B.T.dot(q)
                 w += AP.solve(h)
                 return np.hstack([w, q])
         return pmult
     def getVelocitySolver(self, A):
-        return solvers.cfd.VelcoitySolver(A)
+        return solvers.cfd.VelcoitySolver(A, disp=0, counter="VS", solver=self.precond_v)
+        # return solvers.cfd.VelcoitySolver(A, solver='pyamg', maxiter=1)
     def getPressureSolver(self, A, B, AP):
         mu = self.problemdata.params.scal_glob['mu']
-        return solvers.cfd.PressureSolverDiagonal(self.mesh, mu)    
-    def linearSolver(self, Ain, bin, uin=None, solver='umf', verbose=0, atol=1e-14, rtol=1e-10):
+        if self.pmean: assert self.precond_p == "schur"
+        if self.precond_p[:5] == "schur":
+            sp = self.precond_p.split('@')
+            if not len(sp)==3 or not(0 < int(sp[2]) < 20) or not sp[1] in solvers.cfd.prec_PressureSolverSchur:
+                raise ValueError(f"need 'schur@prec@maxiter' with prec in {solvers.cfd.prec_PressureSolverSchur}\ngot {self.precond_p}" )
+            return solvers.cfd.PressureSolverSchur(self.mesh, mu, A, B, AP, solver='lgmres', prec = sp[1], maxiter=int(sp[2]), disp=0)
+        elif self.precond_p == "diag":    
+            return solvers.cfd.PressureSolverDiagonal(A, B, prec='scale', accel='cg', maxiter=5, disp=1, counter="PS", symmetric=True)
+        elif self.precond_p == "scale":    
+            return solvers.cfd.PressureSolverScale(self.mesh, mu)
+        else:
+            raise ValueError(f"unknown {self.precond_p=}")   
+    def linearSolver(self, Ain, bin, uin=None, linearsolver='umf', verbose=0, atol=1e-14, rtol=1e-10):
         ncells, nfaces, ncomp = self.mesh.ncells, self.mesh.nfaces, self.ncomp
-        if solver == 'umf':
+        if linearsolver == 'umf':
             Aall = self._to_single_matrix(Ain)
             uall =  splinalg.spsolve(Aall, bin, permc_spec='COLAMD')
+            self.timer.add("linearsolve")
             return uall, 1
-        elif solver[:4] == 'iter':
-            ssolver = solver.split('_')
-            method=ssolver[1] if len(ssolver)>1 else 'lgmres'
-            disp=int(ssolver[2]) if len(ssolver)>2 else 0
+        else:
+            # print(f"{atol=} {rtol=}")
+            ssolver = linearsolver.split('_')
+            method=ssolver[0] if len(ssolver)>0 else 'lgmres'
+            disp=int(ssolver[1]) if len(ssolver)>1 else 0
             nall = ncomp*nfaces + ncells
             if self.pmean: nall += 1
-            matvec = partial(self.matrixVector, Ain)
             AP = self.getVelocitySolver(Ain[0])
             SP = self.getPressureSolver(Ain[0], Ain[1], AP)
-            matvecprec=self.getPrecMult(Ain, AP, SP)
-            S = solvers.cfd.SystemSolver(n=nall, matvec=matvec, matvecprec=matvecprec, method=method, disp=disp, atol=atol, rtol=rtol)
-            return S.solve(b=bin, x0=uin)
-        else:
-            raise ValueError(f"unknown solve '{solver=}'")
+            matvec = partial(self.matrixVector, Ain)
+            matvecprec = self.getPrecMult(Ain, AP, SP)
+            maxiter = 20
+            S = solvers.linalg.ScipySolve(matvec=matvec, matvecprec=matvecprec, method=method, n=nall,
+                                            disp=disp, atol=atol, rtol=rtol, counter="sys", maxiter=maxiter)
+            uall =  S.solve(b=bin, x0=uin)
+            self.timer.add("linearsolve")
+            it = S.counter.niter
+            if it==maxiter: 
+                msg = f"*** linear system solver not converged in {maxiter=}"
+                msg += f"\n{self.precond_v=} {self.precond_p=}"
+                msg += f"\n{AP=} {SP=}"
+                msg += f"\n{S.counter=}"
+                raise ValueError(msg)
+            return uall, it
     def computeRhs(self, b=None, u=None, coeffmass=None):
         b = self._zeros()
         bs  = self._split(b)
         bv,bp = bs[0], bs[1]
-        # if self.pmean: bv, bp, bmean = self._split(b)
-        # else: bv, bp = self._split(b)
-        # bv = np.zeros(self.femv.nunknowns() * self.ncomp)
-        # bp = np.zeros(self.femp.nunknowns())
         if 'rhs' in self.problemdata.params.fct_glob:
             rhsv, rhsp = self.problemdata.params.fct_glob['rhs']
             if rhsv: self.femv.computeRhsCells(bv, rhsv)
@@ -252,13 +328,60 @@ class Stokes(Application):
         colorsdir = self.problemdata.bdrycond.colorsOfType("Dirichlet")
         colorsneu = self.problemdata.bdrycond.colorsOfType("Neumann")
         colorsnav = self.problemdata.bdrycond.colorsOfType("Navier")
+        colorsp = self.problemdata.bdrycond.colorsOfType("Pressure")
         self.femv.computeRhsBoundary(bv, colorsneu, self.problemdata.bdrycond.fct)
         if self.dirichletmethod=='strong':
-            self.vectorBoundary((bv, bp), self.problemdata.bdrycond.fct, self.bdrydata, self.dirichletmethod)
+            self.vectorBoundaryStrong((bv, bp), self.problemdata.bdrycond.fct, self.bdrydata, self.dirichletmethod)
         else:
             vdir = self.femv.interpolateBoundary(colorsdir, self.problemdata.bdrycond.fct)
             self.computeRhsBdryNitscheDirichlet((bv,bp), colorsdir, vdir, self.mucell)
-            self.computeRhsBdryNitscheNavier((bv,bp), colorsnav, vdir, self.mucell)
+            bdryfct = self.problemdata.bdrycond.fct
+            colorspres = set(bdryfct.keys()).intersection(colorsnav)
+            if len(colorspres):
+                if not isinstance(bdryfct[next(iter(colorspres))],dict):
+                    msg = """
+                    For Navier b.c. please give a dictionary {vn:fct_scal, g:fvt_vec} with fct_scal scalar and fvt_vec a list of dim functions
+                    """
+                    raise ValueError(msg+f"\ngiven: {bdryfct[next(iter(colorspres))]=}")
+                vnfct, gfct = {}, {}
+                for col in colorspres:
+                    if 'vn' in bdryfct[col].keys() : 
+                        if not callable(bdryfct[col]['vn']):
+                            raise ValueError(f"'vn' must be a function. Given:{bdryfct[col]['vn']=}")
+                        vnfct[col] = bdryfct[col]['vn']
+                    if 'g' in bdryfct[col].keys() : 
+                        if not isinstance(bdryfct[col]['g'], list) or len(bdryfct[col]['g'])!=self.ncomp:
+                            raise ValueError(f"'g' must be a list of functions with {self.ncomp} elements. Given:{bdryfct[col]['g']=}")
+                        gfct[col] = bdryfct[col]['g']
+                if len(vnfct): 
+                    vn = self.femv.fem.interpolateBoundary(colorsnav, vnfct, lumped=False)
+                    self.computeRhsBdryNitscheNavierNormal((bv,bp), colorsnav, self.mucell, vn)
+                if len(gfct): 
+                    gt = self.femv.interpolateBoundary(colorsnav, gfct)
+                    self.computeRhsBdryNitscheNavierTangent((bv,bp), colorsnav, self.mucell, gt)
+            colorspres = set(bdryfct.keys()).intersection(colorsp)
+            if len(colorspres):
+                if not isinstance(bdryfct[next(iter(colorspres))],dict):
+                    msg = """
+                    For Pressure b.c. please give a dictionary {p:fct_scal, v:fvt_vec} with fct_scal scalar and fvt_vec a list of dim functions
+                    """
+                    raise ValueError(msg+f"\ngiven: {bdryfct[next(iter(colorspres))]=}")
+                pfct, vfct = {}, {}
+                for col in colorspres:
+                    if 'p' in bdryfct[col].keys() : 
+                        if not callable(bdryfct[col]['p']):
+                            raise ValueError(f"'vn' must be a function. Given:{bdryfct[col]['p']=}")
+                        pfct[col] = bdryfct[col]['p']
+                    if 'v' in bdryfct[col].keys() : 
+                        if not isinstance(bdryfct[col]['v'], list) or len(bdryfct[col]['v'])!=self.ncomp:
+                            raise ValueError(f"'v' must be a list of functions with {self.ncomp} elements. Given:{bdryfct[col]['v']=}")
+                        vfct[col] = bdryfct[col]['v']
+                if len(pfct):
+                    p = self.femv.fem.interpolateBoundary(colorsp, pfct, lumped=False)
+                    self.computeRhsBdryNitschePressureNormal((bv,bp), colorsp, self.mucell, p)
+                if len(vfct): 
+                    v = self.femv.interpolateBoundary(colorsp, vfct)
+                    self.computeRhsBdryNitschePressureTangent((bv,bp), colorsp, self.mucell, v)
         if not self.pmean: return b
         if self.problemdata.solexact is not None:
             p = self.problemdata.solexact[1]
@@ -278,10 +401,12 @@ class Stokes(Application):
         self.femv.computeFormLaplace(self.mucell, dv, v)
         self.femv.computeFormDivGrad(dv, dp, v, p)
         colorsdir = self.problemdata.bdrycond.colorsOfType("Dirichlet")
+        colorsnav = self.problemdata.bdrycond.colorsOfType("Navier")
         if self.dirichletmethod == 'strong':
             self.femv.formBoundary(dv, self.bdrydata, self.dirichletmethod)
         else:
-            self.computeFormBdryNitsche(dv, dp, v, p, colorsdir, self.mucell)
+            self.computeFormBdryNitscheDirichlet(dv, dp, v, p, colorsdir, self.mucell)
+            self.computeFormBdryNitscheNavier(dv, dp, v, p, colorsnav, self.mucell)
         if self.pmean:
             self.computeFormMeanPressure(dp, dlam, p, lam)
         # if not np.allclose(d,d2):
@@ -292,12 +417,21 @@ class Stokes(Application):
         B = self.femv.computeMatrixDivergence()
         colorsdir = self.problemdata.bdrycond.colorsOfType("Dirichlet")
         colorsnav = self.problemdata.bdrycond.colorsOfType("Navier")
+        colorsp = self.problemdata.bdrycond.colorsOfType("Pressure")
         if self.dirichletmethod == 'strong':
-            A, B = self.matrixBoundary(A, B, self.bdrydata, self.dirichletmethod)
+            A, B = self.matrixBoundaryStrong(A, B, self.bdrydata, self.dirichletmethod)
         else:
             #TODO eviter le retour de A,B
+            # print(f"{id(A)=} {id(B)=}")
             A, B = self.computeMatrixBdryNitscheDirichlet(A, B, colorsdir, self.mucell)
-            A, B = self.computeMatrixBdryNitscheNavier(A, B, colorsnav, self.mucell)
+            # print(f"{id(A)=} {id(B)=}")
+            lam = self.problemdata.params.scal_glob.get('navier',0) 
+            A, B = self.computeMatrixBdryNitscheNavier(A, B, colorsnav, self.mucell, lam)
+            A, B = self.computeMatrixBdryNitschePressure(A, B, colorsp, self.mucell)
+            # print(f"{id(A)=} {id(B)=}")
+        if self.hdivpenalty:
+            if not hasattr(self.mesh,'innerfaces'): self.mesh.constructInnerFaces()
+            A += self.femv.computeMatrixRtPenaly(self.hdivpenalty)
         if not self.pmean:
             return [A, B]
         ncells = self.mesh.ncells
@@ -309,80 +443,53 @@ class Stokes(Application):
         dlam += self.mesh.dV.dot(p)
         dp += lam*self.mesh.dV
     def computeBdryNormalFluxNitsche(self, v, p, colors):
-        nfaces, ncells, ncomp = self.mesh.nfaces, self.mesh.ncells, self.ncomp
-        bdryfct = self.problemdata.bdrycond.fct
-        flux, omega = np.zeros(shape=(len(colors),ncomp)), np.zeros(len(colors))
-        xf, yf, zf = self.mesh.pointsf.T
-        cellgrads = self.femv.fem.cellgrads
-        facesOfCell = self.mesh.facesOfCells
-        mucell = self.mucell
-        for i,color in enumerate(colors):
-            faces = self.mesh.bdrylabels[color]
-            cells = self.mesh.cellsOfFaces[faces,0]
-            normalsS = self.mesh.normals[faces][:,:ncomp]
-            dS = np.linalg.norm(normalsS, axis=1)
-            if color in bdryfct:
-                # bfctv, bfctp = bdryfct[color]
-                bfctv = bdryfct[color]
-                # dirichv = np.hstack([bfctv(xf[faces], yf[faces], zf[faces])])
-                dirichv = np.vstack([f(xf[faces], yf[faces], zf[faces]) for f in bfctv])
-            flux[i] -= np.einsum('f,fk->k', p[cells], normalsS)
-            indfaces = self.mesh.facesOfCells[cells]
-            ind = npext.positionin(faces, indfaces).astype(int)
-            for icomp in range(ncomp):
-                vicomp = v[icomp+ ncomp*facesOfCell[cells]]
-                flux[i,icomp] = np.einsum('fj,f,fi,fji->', vicomp, mucell[cells], normalsS, cellgrads[cells, :, :ncomp])
-                vD = v[icomp+ncomp*faces]
-                if color in bdryfct:
-                    vD -= dirichv[icomp]
-                flux[i,icomp] -= self.dirichlet_nitsche*np.einsum('f,fi,fi->', vD * mucell[cells], normalsS, cellgrads[cells, ind, :ncomp])
-            # for icomp in range(ncomp):
-            #     mat2 = np.einsum('fj,f->fj', mat, dirichv[icomp])
-            #     np.add.at(bv, icomp + ncomp * indfaces, -mat2)
-            # ind = npext.positionin(faces, indfaces).astype(int)
-            # for icomp in range(ncomp):
-            #     bv[icomp + ncomp * faces] += self.dirichlet_nitsche * np.choose(ind, mat.T) * dirichv[icomp]
-        return flux.T
-    def computeRhsBdryNitscheDirichlet(self, b, colorsdir, vdir, mucell, coeff=1):
+        ncomp, bdryfct = self.ncomp, self.problemdata.bdrycond.fct
+        flux = np.zeros(shape=(ncomp,len(colors)))
+        vdir = self.femv.interpolateBoundary(colors, bdryfct).ravel()
+        for icomp in range(ncomp):
+            flux[icomp] = self.femv.fem.computeBdryNormalFluxNitsche(v[icomp::ncomp], colors, vdir[icomp::ncomp], self.mucell, nitsche_param=self.dirichlet_nitsche)
+            for i,color in enumerate(colors):
+                faces = self.mesh.bdrylabels[color]
+                cells = self.mesh.cellsOfFaces[faces,0]
+                normalsS = self.mesh.normals[faces][:,:ncomp]
+                dS = np.linalg.norm(normalsS, axis=1)
+                flux[icomp,i] -= p[cells].dot(normalsS[:,icomp])
+        return flux
+    def computeRhsBdryNitscheDirichlet(self, b, colors, vdir, mucell, coeff=1):
         bv, bp = b
         ncomp  = self.ncomp
-        faces = self.mesh.bdryFaces(colorsdir)
+        faces = self.mesh.bdryFaces(colors)
         cells = self.mesh.cellsOfFaces[faces,0]
         normalsS = self.mesh.normals[faces][:,:ncomp]
         np.add.at(bp, cells, -np.einsum('nk,nk->n', coeff*vdir[faces], normalsS))
-        self.femv.computeRhsNitscheDiffusion(bv, mucell, colorsdir, vdir, ncomp)
-    def computeRhsBdryNitscheNavier(self, b, colors, vdir, mucell, coeff=1):
-        if len(colors): raise NotImplementedError("trop tot")
-    def computeRhsBdryNitscheOld(self, b, colorsdir, bdryfct, mucell, coeff=1):
+        self.femv.computeRhsNitscheDiffusion(bv, mucell, colors, vdir, ncomp, nitsche_param=self.dirichlet_nitsche)
+    def computeRhsBdryNitscheNavierNormal(self, b, colors, mucell, vn):
         bv, bp = b
-        xf, yf, zf = self.mesh.pointsf.T
-        nfaces, ncells, dim, ncomp  = self.mesh.nfaces, self.mesh.ncells, self.mesh.dimension, self.ncomp
-        cellgrads = self.femv.fem.cellgrads
-        for color in colorsdir:
-            faces = self.mesh.bdrylabels[color]
-            cells = self.mesh.cellsOfFaces[faces,0]
-            normalsS = self.mesh.normals[faces][:,:ncomp]
-            dS = np.linalg.norm(normalsS,axis=1)
-            # normalsS = normalsS/dS[:,np.newaxis]
-            if not color in bdryfct.keys(): continue
-            bfctv = bdryfct[color]
-            # dirichv = np.hstack([bfctv(xf[faces], yf[faces], zf[faces])])
-            dirichv = np.vstack([f(xf[faces], yf[faces], zf[faces]) for f in bfctv])
-            # print(f"{dirichv.shape=} {normalsS.shape=}")
-            bp[cells] -= np.einsum('kn,nk->n', coeff*dirichv, normalsS)
-            mat = np.einsum('f,fi,fji->fj', coeff*mucell[cells], normalsS, cellgrads[cells, :, :dim])
-            indfaces = self.mesh.facesOfCells[cells]
-            for icomp in range(ncomp):
-                mat2 = np.einsum('fj,f->fj', mat, dirichv[icomp])
-                np.add.at(bv, icomp+ncomp*indfaces, -mat2)
-            ind = npext.positionin(faces, indfaces).astype(int)
-            for icomp in range(ncomp):
-                bv[icomp+ncomp*faces] += self.dirichlet_nitsche * np.choose(ind, mat.T)*dirichv[icomp]
-        # print(f"{bv.shape=} {bp.shape=}")
-        if len(colors): raise NotImplementedError("trop tot")
-    def computeFormBdryNitsche(self, dv, dp, v, p, colorsdir, mu):
+        ncomp, dim  = self.ncomp, self.mesh.dimension
+        faces = self.mesh.bdryFaces(colors)
+        cells = self.mesh.cellsOfFaces[faces,0]
+        normalsS = self.mesh.normals[faces][:,:ncomp]
+        dS = np.linalg.norm(normalsS, axis=1)
+        # normals = normalsS/dS[:,np.newaxis]
+        # foc = self.mesh.facesOfCells[cells]
+        np.add.at(bp, cells, -dS*vn[faces])
+        self.femv.computeRhsNitscheDiffusionNormal(bv, mucell, colors, vn, ncomp, nitsche_param=self.dirichlet_nitsche)
+    def computeRhsBdryNitscheNavierTangent(self, b, colors, mucell, gt):
+        bv, bp = b
+        ncomp, dim  = self.ncomp, self.mesh.dimension
+        self.femv.massDotBoundary(bv, gt.ravel(), colors=colors, ncomp=ncomp, coeff=1)
+        self.femv.massDotBoundaryNormal(bv, -gt.ravel(), colors=colors, ncomp=ncomp, coeff=1)
+    def computeRhsBdryNitschePressureNormal(self, b, colors, mucell, p):
+        bv, bp = b
+        self.femv.massDotBoundaryNormal(bv, -p, colors=colors, ncomp=self.ncomp, coeff=1)
+    def computeRhsBdryNitschePressureTangent(self, b, colors, mucell, v):
+        bv, bp = b
+        ncomp, dim  = self.ncomp, self.mesh.dimension
+        self.femv.computeRhsNitscheDiffusion(bv, mucell, colors, v, ncomp, nitsche_param=self.dirichlet_nitsche)
+        self.femv.computeRhsNitscheDiffusionNormal(bv, mucell, colors, -v.ravel(), ncomp, nitsche_param=self.dirichlet_nitsche)
+    def computeFormBdryNitscheDirichlet(self, dv, dp, v, p, colorsdir, mu):
         ncomp, dim  = self.femv.ncomp, self.mesh.dimension
-        self.femv.computeFormNitscheDiffusion(dv, v, mu, colorsdir, ncomp)
+        self.femv.computeFormNitscheDiffusion(dv, v, mu, colorsdir, ncomp, nitsche_param=self.dirichlet_nitsche)
         faces = self.mesh.bdryFaces(colorsdir)
         cells = self.mesh.cellsOfFaces[faces, 0]
         normalsS = self.mesh.normals[faces][:, :self.ncomp]
@@ -391,10 +498,14 @@ class Stokes(Application):
             np.add.at(dv[icomp::ncomp], faces, r)
             r = np.einsum('f,f->f', normalsS[:,icomp], v[icomp::ncomp][faces])
             np.add.at(dp, cells, -r)
-    def computeMatrixBdryNitscheDirichlet(self, A, B, colorsdir, mucell):
+    def computeFormBdryNitscheNavier(self, dv, dp, v, p, colors, mu):
+        if not len(colors): return
+        raise NotImplementedError()
+    def computeMatrixBdryNitscheDirichlet(self, A, B, colors, mucell):
         nfaces, ncells, ncomp, dim  = self.mesh.nfaces, self.mesh.ncells, self.femv.ncomp, self.mesh.dimension
-        A += self.femv.computeMatrixNitscheDiffusion(mucell, colorsdir, ncomp)
-        faces = self.mesh.bdryFaces(colorsdir)
+        A += self.femv.computeMatrixNitscheDiffusion(mucell, colors, ncomp, nitsche_param=self.dirichlet_nitsche)
+        #grad-div
+        faces = self.mesh.bdryFaces(colors)
         cells = self.mesh.cellsOfFaces[faces, 0]
         normalsS = self.mesh.normals[faces][:, :self.ncomp]
         indfaces = np.repeat(ncomp * faces, ncomp)
@@ -402,49 +513,31 @@ class Stokes(Application):
         cols = indfaces.ravel()
         rows = cells.repeat(ncomp).ravel()
         mat = normalsS.ravel()
-        BN = sparse.coo_matrix((mat, (rows, cols)), shape=(ncells, ncomp*nfaces)).tocsr()
-        B -= BN
+        B -= sparse.coo_matrix((mat, (rows, cols)), shape=(ncells, ncomp*nfaces))
         return A,B
-    def computeMatrixBdryNitscheNavier(self, A, B, colors, mucell):
-        if len(colors): raise NotImplementedError("trop tot")
-        return A,B
-    def computeMatrixBdryNitscheOld(self, A, B, colorsdir, mucell):
+    def computeMatrixBdryNitscheNavier(self, A, B, colors, mucell, lambdaR):
         nfaces, ncells, ncomp, dim  = self.mesh.nfaces, self.mesh.ncells, self.femv.ncomp, self.mesh.dimension
-        A += self.femv.computeMatrixNitscheDiffusion(mucell, colorsdir, ncomp)
-        # nloc = dim+1
-        # nlocncomp = ncomp * nloc
-        # cellgrads = self.femv.fem.cellgrads
-        faces = self.mesh.bdryFaces(colorsdir)
+        faces = self.mesh.bdryFaces(colors)
         cells = self.mesh.cellsOfFaces[faces, 0]
-        normalsS = self.mesh.normals[faces][:, :self.ncomp]
+        normalsS = self.mesh.normals[faces][:, :dim]
+        #grad-div
         indfaces = np.repeat(ncomp * faces, ncomp)
         for icomp in range(ncomp): indfaces[icomp::ncomp] += icomp
         cols = indfaces.ravel()
         rows = cells.repeat(ncomp).ravel()
-        mat = normalsS.ravel()
-        BN = sparse.coo_matrix((mat, (rows, cols)), shape=(ncells, ncomp*nfaces)).tocsr()
-
-        # mat = np.einsum('f,fi,fji->fj', mucell[cells], normalsS, cellgrads[cells, :, :dim])
-        # mats = mat.repeat(ncomp)
-        # dofs = self.mesh.facesOfCells[cells, :]
-        # cols = np.repeat(ncomp * dofs, ncomp).reshape(dofs.shape[0], nloc, ncomp)
-        # for icomp in range(ncomp): cols[:,:,icomp::ncomp] += icomp
-        # indfaces = np.repeat(ncomp * faces, ncomp*nloc).reshape(dofs.shape[0],nloc,ncomp)
-        # for icomp in range(ncomp): indfaces[:,:,icomp::ncomp] += icomp
-        # cols = cols.reshape(dofs.shape[0],nloc*ncomp)
-        # rows = indfaces.reshape(dofs.shape[0],nloc*ncomp)
-        # AN = sparse.coo_matrix((mats, (rows.ravel(), cols.ravel())), shape=(ncomp*nfaces, ncomp*nfaces)).tocsr()
-        # AD = sparse.diags(AN.diagonal(), offsets=(0), shape=(ncomp*nfaces, ncomp*nfaces))
-        # A = A- AN -AN.T + self.dirichlet_nitsche*AD
-        # print(f"{AN.todense()=}")
-        # print(f"{AD.toarray()=}")
-        # print(f"{A.toarray()=}")
-        # print(f"{A=}")
-        B -= BN
+        B -= sparse.coo_matrix((normalsS.ravel(), (rows, cols)), shape=(ncells, ncomp*nfaces))
+        #vitesses
+        A += self.femv.computeMatrixNitscheDiffusionNormal(mucell, colors, ncomp, nitsche_param=self.dirichlet_nitsche)
+        A += self.femv.computeMassMatrixBoundary(colors, ncomp, coeff=lambdaR)-self.femv.computeMassMatrixBoundaryNormal(colors, ncomp, coeff=lambdaR)
         return A,B
-    def vectorBoundary(self, b, bdryfctv, bdrydata, method):
+    def computeMatrixBdryNitschePressure(self, A, B, colors, mucell):
+        #vitesses
+        A += self.femv.computeMatrixNitscheDiffusion(mucell, colors, self.ncomp, nitsche_param=self.dirichlet_nitsche)
+        A -= self.femv.computeMatrixNitscheDiffusionNormal(mucell, colors, self.ncomp, nitsche_param=self.dirichlet_nitsche)
+        return A,B
+    def vectorBoundaryStrong(self, b, bdryfctv, bdrydata, method):
         bv, bp = b
-        bv = self.femv.vectorBoundary(bv, bdryfctv, bdrydata, method)
+        bv = self.femv.vectorBoundaryStrong(bv, bdryfctv, bdrydata, method)
         facesdirall, facesinner, colorsdir, facesdirflux = bdrydata.facesdirall, bdrydata.facesinner, bdrydata.colorsdir, bdrydata.facesdirflux
         nfaces, ncells, ncomp  = self.mesh.nfaces, self.mesh.ncells, self.femv.ncomp
         bdrydata.bsaved = {}
@@ -457,8 +550,8 @@ class Stokes(Application):
         #suppose strong-trad
         bp -= bdrydata.B_inner_dir * bv[inddir]
         return (bv,bp)
-    def matrixBoundary(self, A, B, bdrydata, method):
-        A = self.femv.matrixBoundary(A, bdrydata, method)
+    def matrixBoundaryStrong(self, A, B, bdrydata, method):
+        A = self.femv.matrixBoundaryStrong(A, bdrydata, method)
         facesdirall, facesinner, colorsdir, facesdirflux = bdrydata.facesdirall, bdrydata.facesinner, bdrydata.colorsdir, bdrydata.facesdirflux
         nfaces, ncells, ncomp  = self.mesh.nfaces, self.mesh.ncells, self.femv.ncomp
         bdrydata.Bsaved = {}
