@@ -17,7 +17,7 @@ from simfempy.meshes import move, plotmesh
 class CR1(fems.fem.Fem):
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
-        self.dirichlet_al = 1
+        self.dirichlet_strong = 1
         self.dirichlet_nitsche = 2
     def setMesh(self, mesh):
         super().setMesh(mesh)
@@ -86,28 +86,8 @@ class CR1(fems.fem.Fem):
             facesdir = self.mesh.bdrylabels[color]
             bdrydata.facesdirflux[color] = facesdir
         return bdrydata
-    def computeRhsNitscheDiffusionOld(self, b, diffcoff, colorsdir, bdrycond, coeff=1):
-        #TODO Nitsche sdtab term ???
-        nfaces, ncells, dim, nlocal  = self.mesh.nfaces, self.mesh.ncells, self.mesh.dimension, self.nlocal()
-        x, y, z = self.mesh.pointsf.T
-        for color in colorsdir:
-            faces = self.mesh.bdrylabels[color]
-            cells = self.mesh.cellsOfFaces[faces,0]
-            normalsS = self.mesh.normals[faces][:,:dim]
-            dS = np.linalg.norm(normalsS,axis=1)
-            if not color in bdrycond.fct: continue
-            dirichlet = bdrycond.fct[color]
-            u = dirichlet(x[faces], y[faces], z[faces])
-            mat = np.einsum('f,fi,fji->fj', coeff*u*diffcoff[cells], normalsS, self.cellgrads[cells, :, :dim])
-            np.add.at(b, self.mesh.facesOfCells[cells], -mat)
-            ind = npext.positionin(faces, self.mesh.facesOfCells[cells]).astype(int)
-            if not np.all(faces == self.mesh.facesOfCells[cells,ind]):
-                print(f"{faces=}")
-                print(f"{self.mesh.facesOfCells[cells]=}")
-                print(f"{self.mesh.facesOfCells[cells,ind]=}")
-            b[faces] += self.dirichlet_nitsche * np.choose(ind,mat.T)
-        return b
-    def computeRhsNitscheDiffusion(self, b, diffcoff, colorsdir, udir, coeff=1):
+    def computeRhsNitscheDiffusion(self, b, diffcoff, colorsdir, udir, nitsche_param=None, coeff=1, lumped=False):
+        if nitsche_param==None: nitsche_param=self.dirichlet_nitsche
         assert udir.shape[0] == self.mesh.nfaces
         dim, faces = self.mesh.dimension, self.mesh.bdryFaces(colorsdir)
         cells = self.mesh.cellsOfFaces[faces,0]
@@ -115,8 +95,9 @@ class CR1(fems.fem.Fem):
         dS, dV = np.linalg.norm(normalsS,axis=1), self.mesh.dV[cells]
         mat = np.einsum('f,fi,fji->fj', coeff*udir[faces]*diffcoff[cells], normalsS, self.cellgrads[cells, :, :dim])
         np.add.at(b, self.mesh.facesOfCells[cells], -mat)
-        self.massDotBoundary(b, f=udir, colors=colorsdir, coeff=self.dirichlet_nitsche * dS/dV)
-    def computeFormNitscheDiffusion(self, du, u, diffcoff, colorsdir):
+        self.massDotBoundary(b, f=udir, colors=colorsdir, coeff=coeff*nitsche_param*diffcoff[cells] * dS/dV, lumped=lumped)
+    def computeFormNitscheDiffusion(self, du, u, diffcoff, colorsdir, nitsche_param=None):
+        if nitsche_param==None: nitsche_param=self.dirichlet_nitsche
         assert u.shape[0] == self.mesh.nfaces
         dim, faces = self.mesh.dimension, self.mesh.bdryFaces(colorsdir)
         cells = self.mesh.cellsOfFaces[faces,0]
@@ -126,44 +107,27 @@ class CR1(fems.fem.Fem):
         np.add.at(du, foc, -mat)
         mat = np.einsum('f,fk,fjk,fj->f', diffcoff[cells], normalsS, cellgrads, u[foc])
         np.add.at(du, faces, -mat)
-        self.massDotBoundary(du, f=u, colors=colorsdir, coeff=self.dirichlet_nitsche * dS/dV)
-    def computeMatrixNitscheDiffusion(self, diffcoff, colorsdir, coeff=1):
+        self.massDotBoundary(du, f=u, colors=colorsdir, coeff=nitsche_param*diffcoff[cells]* dS/dV)
+    def computeMatrixNitscheDiffusion(self, diffcoff, colors, nitsche_param=None, coeff=1, lumped=False):
+        if nitsche_param==None: nitsche_param=self.dirichlet_nitsche
         nfaces, ncells, dim, nlocal  = self.mesh.nfaces, self.mesh.ncells, self.mesh.dimension, self.nlocal()
-        faces = self.mesh.bdryFaces(colorsdir)
+        faces = self.mesh.bdryFaces(colors)
+        if not isinstance(coeff, (float,int)): assert coeff.shape[0]==faces.shape[0]
         cells = self.mesh.cellsOfFaces[faces, 0]
         normalsS = self.mesh.normals[faces][:, :dim]
         cols = self.mesh.facesOfCells[cells, :].ravel()
         rows = faces.repeat(nlocal)
-        mat = np.einsum('f,fi,fji->fj', coeff * diffcoff[cells], normalsS, self.cellgrads[cells, :, :dim]).ravel()
+        mat = np.einsum('f,fi,fji->fj', coeff*diffcoff[cells], normalsS, self.cellgrads[cells, :, :dim]).ravel()
         AN = sparse.coo_matrix((mat, (rows, cols)), shape=(nfaces, nfaces)).tocsr()
         # AD = sparse.diags(AN.diagonal(), offsets=(0), shape=(nfaces, nfaces))
         dS = np.linalg.norm(normalsS,axis=1)
         dV = self.mesh.dV[cells]
         # AD = sparse.coo_matrix((dS**2/dV,(faces,faces)), shape=(nfaces, nfaces))
-        AD = self.computeBdryMassMatrix(colors=colorsdir, coeff=dS/dV)
-        return - AN -AN.T + self.dirichlet_nitsche*AD
-    def computeBdryNormalFluxNitscheOld(self, u, colors, bdrycond, diffcoff):
-        flux= np.zeros(len(colors))
-        nfaces, ncells, dim, nlocal  = self.mesh.nfaces, self.mesh.ncells, self.mesh.dimension, self.nlocal()
-        facesOfCell = self.mesh.facesOfCells
-        x, y, z = self.mesh.pointsf.T
-        for i,color in enumerate(colors):
-            faces = self.mesh.bdrylabels[color]
-            cells = self.mesh.cellsOfFaces[faces, 0]
-            normalsS = self.mesh.normals[faces,:dim]
-            cellgrads = self.cellgrads[cells, :, :dim]
-            # print(f"{u[facesOfCell[cells]].shape=}")
-            # print(f"{normalsS.shape=}")
-            flux[i] = np.einsum('fj,f,fi,fji->', u[facesOfCell[cells]], diffcoff[cells], normalsS, cellgrads)
-            dirichlet = bdrycond.fct[color]
-            uD = u[faces]
-            if color in bdrycond.fct:
-                uD -= dirichlet(x[faces], y[faces], z[faces])
-            ind = npext.positionin(faces, self.mesh.facesOfCells[cells]).astype(int)
-            # print(f"{self.cellgrads[cells, ind, :dim].shape=}")
-            flux[i] -= self.dirichlet_nitsche*np.einsum('f,fi,fi->', uD * diffcoff[cells], normalsS, self.cellgrads[cells, ind, :dim])
-        return flux
-    def computeBdryNormalFluxNitsche(self, u, colors, udir, diffcoff):
+        AD = self.computeBdryMassMatrix(colors=colors, coeff=coeff*diffcoff[cells]*nitsche_param*dS/dV, lumped=lumped)
+        return AD - AN -AN.T 
+    def computeBdryNormalFluxNitsche(self, u, colors, udir, diffcoff, nitsche_param=None):
+        if nitsche_param==None: nitsche_param=self.dirichlet_nitsche
+        #TODO correct flux computation Nitsche
         flux= np.zeros(len(colors))
         nfaces, ncells, dim, nlocal  = self.mesh.nfaces, self.mesh.ncells, self.mesh.dimension, self.nlocal()
         facesOfCell = self.mesh.facesOfCells
@@ -172,17 +136,19 @@ class CR1(fems.fem.Fem):
             cells = self.mesh.cellsOfFaces[faces, 0]
             normalsS = self.mesh.normals[faces,:dim]
             cellgrads = self.cellgrads[cells, :, :dim]
-            flux[i] = np.einsum('fj,f,fi,fji->', u[facesOfCell[cells]], diffcoff[cells], normalsS, cellgrads)
+            foc = facesOfCell[cells]
+            flux[i] = np.einsum('fj,f,fi,fji->', u[foc], diffcoff[cells], normalsS, cellgrads)
             dS = np.linalg.norm(normalsS,axis=1)
             dV = self.mesh.dV[cells]
-            flux[i] -= self.massDotBoundary(b=None, f=u-udir, colors=[color], coeff=self.dirichlet_nitsche * dS/dV)
+            flux[i] -= self.massDotBoundary(b=None, f=u-udir, colors=[color], coeff=nitsche_param * diffcoff[cells]*dS/dV)
+            # flux[i] /= np.sum(dS)
         return flux
-    def vectorBoundaryEqual(self, du, u, bdrydata):
+    def vectorBoundaryStrongEqual(self, du, u, bdrydata):
         facesdirall = bdrydata.facesdirall
         du[facesdirall] = u[facesdirall]
-    def vectorBoundaryZero(self, du, bdrydata):
+    def vectorBoundaryStrongZero(self, du, bdrydata):
         du[bdrydata.facesdirall] = 0
-    def vectorBoundary(self, b, bdrycond, bdrydata, method):
+    def vectorBoundaryStrong(self, b, bdrycond, bdrydata, method):
         facesdirflux, facesinner, facesdirall, colorsdir = bdrydata.facesdirflux, bdrydata.facesinner, bdrydata.facesdirall, bdrydata.colorsdir
         x, y, z = self.mesh.pointsf.T
         for color, faces in facesdirflux.items():
@@ -198,7 +164,7 @@ class CR1(fems.fem.Fem):
             b[facesdirall] = help[facesdirall]
         else:
             b[facesdirall] = bdrydata.A_dir_dir * help[facesdirall]
-    def matrixBoundary(self, A, bdrydata, method):
+    def matrixBoundaryStrong(self, A, bdrydata, method):
         assert method != 'nitsche'
         facesdirflux, facesinner, facesdirall, colorsdir = bdrydata.facesdirflux, bdrydata.facesinner, bdrydata.facesdirall, bdrydata.colorsdir
         nfaces = self.mesh.nfaces
@@ -217,8 +183,8 @@ class CR1(fems.fem.Fem):
             diag[facesdirall] = 1.0
             diag = sparse.dia_matrix((diag, 0), shape=(nfaces, nfaces))
         else:
-            bdrydata.A_dir_dir = self.dirichlet_al*A[facesdirall, :][:, facesdirall]
-            diag[facesdirall] = np.sqrt(self.dirichlet_al)
+            bdrydata.A_dir_dir = self.dirichlet_strong*A[facesdirall, :][:, facesdirall]
+            diag[facesdirall] = np.sqrt(self.dirichlet_strong)
             diag = sparse.dia_matrix((diag, 0), shape=(nfaces, nfaces))
             diag = diag.dot(A.dot(diag))
         A = help.dot(A)
@@ -264,10 +230,15 @@ class CR1(fems.fem.Fem):
             foc = self.mesh.facesOfCells[ci]
             x, y, z = self.mesh.pointsf[foc].T
             nx, ny, nz = normalsS.T
-            try:
+            import inspect
+            # print(f"{len(inspect.signature(f[color]).parameters)=}")
+            # print(f"{str(inspect.signature(f[color]).parameters)=}")
+            # if 'nx' in str(inspect.signature(f[color])):
+            if len(inspect.signature(f[color]).parameters) >= 6:
+                # ff = f[color](x, y, z, nx[None,:], ny[None,:], nz[None,:])
                 ff = f[color](x, y, z, nx, ny, nz)
-            except:
-                ff = f[color](x, y, z)
+            else:
+                ff = np.vectorize(f[color])(x, y, z)
             np.put(b, foc, ff.T)
         return b
     # matrices
@@ -286,7 +257,7 @@ class CR1(fems.fem.Fem):
     def computeBdryMassMatrix(self, colors=None, coeff=1, lumped=False):
         nfaces, dim = self.mesh.nfaces, self.mesh.dimension
         massloc = barycentric.crbdryothers(dim)
-        lumped = False
+        # lumped = False
         if colors is None: colors = self.mesh.bdrylabels.keys()
 
         if not isinstance(coeff, dict):
@@ -295,11 +266,14 @@ class CR1(fems.fem.Fem):
             dS = linalg.norm(normalsS, axis=1)
             if isinstance(coeff, (int,float)): dS *= coeff
             elif coeff.shape[0]==self.mesh.nfaces: dS *= coeff[faces]
-            else: dS *= coeff
+            elif coeff.shape[0]==dS.shape[0]: dS *= coeff
+            else: raise  ValueError(f"cannot handle {coeff=}")
             AD = sparse.coo_matrix((dS, (faces, faces)), shape=(nfaces, nfaces))
+            if lumped: return AD
             ci = self.mesh.cellsOfFaces[faces][:,0]
             foc = self.mesh.facesOfCells[ci]
             mask = foc != faces[:, np.newaxis]
+            # print(f"{mask=}")
             fi = foc[mask].reshape(foc.shape[0], foc.shape[1] - 1)
             # print(f"{massloc=}")
             cols = np.tile(fi, dim).ravel()
@@ -333,11 +307,9 @@ class CR1(fems.fem.Fem):
     def massDotBoundary(self, b=None, f=None, colors=None, coeff=1, lumped=False):
         #TODO CR1 boundary integrals: can do at ones since last index in facesOfCells is the bdry side:
         # assert np.all(faces == foc[:,-1])
-
         if colors is None: colors = self.mesh.bdrylabels.keys()
         massloc = barycentric.crbdryothers(self.mesh.dimension)
-        lumped==False
-
+        # lumped==False
         if not isinstance(coeff, dict):
             faces = self.mesh.bdryFaces(colors)
             normalsS = self.mesh.normals[faces]
@@ -345,16 +317,17 @@ class CR1(fems.fem.Fem):
             if isinstance(coeff, (int,float)): dS *= coeff
             elif coeff.shape[0]==self.mesh.nfaces: dS *= coeff[faces]
             else: dS *= coeff
-            if b is None: bmean = np.mean(dS*f[faces])
+            if b is None: bsum = np.sum(dS*f[faces])
             else: b[faces] += dS*f[faces]
-            ci = self.mesh.cellsOfFaces[faces][:, 0]
-            foc = self.mesh.facesOfCells[ci]
-            mask = foc!=faces[:,np.newaxis]
-            fi = foc[mask].reshape(foc.shape[0],foc.shape[1]-1)
-            r = np.einsum('n,kl,nl->nk', dS, massloc, f[fi])
-            if b is None: return bmean+np.sum(r)
-            # print(f"{np.linalg.norm(f[fi])=}")
-            np.add.at(b, fi, r)
+            if not lumped:
+                ci = self.mesh.cellsOfFaces[faces][:, 0]
+                foc = self.mesh.facesOfCells[ci]
+                mask = foc!=faces[:,np.newaxis]
+                fi = foc[mask].reshape(foc.shape[0],foc.shape[1]-1)
+                r = np.einsum('n,kl,nl->nk', dS, massloc, f[fi])
+                if b is None: return bsum+np.sum(r)
+                # print(f"{np.linalg.norm(f[fi])=}")
+                np.add.at(b, fi, r)
             return b
 
         assert(isinstance(coeff, dict))
@@ -364,6 +337,7 @@ class CR1(fems.fem.Fem):
             dS = linalg.norm(normalsS, axis=1)
             dS *= coeff[color]
             b[faces] += dS*f[faces]
+            if lumped: continue
             ci = self.mesh.cellsOfFaces[faces][:, 0]
             foc = self.mesh.facesOfCells[ci]
             mask = foc!=faces[:,np.newaxis]
@@ -522,9 +496,10 @@ class CR1(fems.fem.Fem):
         A = self.computeMatrixTransportCellWise(data, type='supg')
         A += self.computeMatrixJump(betart)
         return A
-    def computeMatrixTransportLps(self, data):
+    def computeMatrixTransportLps(self, data, **kwargs):
         A = self.computeMatrixTransportCellWise(data, type='centered')
-        A += self.computeMatrixLps(data.beta)
+        A += self.computeMatrixJump(data.betart)
+        A += self.computeMatrixLps(data.betart, **kwargs)
         return A
     def computeMatrixTransportCellWise(self, data, type):
         beta, betart = data.beta, data.betart
@@ -549,10 +524,11 @@ class CR1(fems.fem.Fem):
         normalsS = self.mesh.normals[innerfaces]
         dS = linalg.norm(normalsS, axis=1)
         faces = self.mesh.faces[self.mesh.innerfaces]
-        ind0 = npext.positionin(faces, self.mesh.simplices[ci0])
-        ind1 = npext.positionin(faces, self.mesh.simplices[ci1])
-        fi0 = np.take_along_axis(self.mesh.facesOfCells[ci0], ind0, axis=1)
-        fi1 = np.take_along_axis(self.mesh.facesOfCells[ci1], ind1, axis=1)
+        # ind0 = npext.positionin(faces, self.mesh.simplices[ci0])
+        # ind1 = npext.positionin(faces, self.mesh.simplices[ci1])
+        # fi0 = np.take_along_axis(self.mesh.facesOfCells[ci0], ind0, axis=1)
+        # fi1 = np.take_along_axis(self.mesh.facesOfCells[ci1], ind1, axis=1)
+        fi0, fi1 = self.mesh.facesOfCellsNotOnInnerFaces(ci0, ci1)
         ifaces = np.arange(nfaces)[innerfaces]
         A = sparse.coo_matrix((ndofs, ndofs))
         rows0 = np.repeat(fi0, nloc-1).ravel()
@@ -590,10 +566,12 @@ class CR1(fems.fem.Fem):
         normalsS = self.mesh.normals[innerfaces]
         dS = linalg.norm(normalsS, axis=1)
         faces = self.mesh.faces[self.mesh.innerfaces]
-        ind0 = npext.positionin(faces, self.mesh.simplices[ci0])
-        ind1 = npext.positionin(faces, self.mesh.simplices[ci1])
-        fi0 = np.take_along_axis(self.mesh.facesOfCells[ci0], ind0, axis=1)
-        fi1 = np.take_along_axis(self.mesh.facesOfCells[ci1], ind1, axis=1)
+        # ind0 = npext.positionin(faces, self.mesh.simplices[ci0])
+        # ind1 = npext.positionin(faces, self.mesh.simplices[ci1])
+        # fi0 = np.take_along_axis(self.mesh.facesOfCells[ci0], ind0, axis=1)
+        # fi1 = np.take_along_axis(self.mesh.facesOfCells[ci1], ind1, axis=1)
+        fi0, fi1 = self.mesh.facesOfCellsNotOnInnerFaces(ci0, ci1)
+        # fi0, fi1 = self.mesh.facesOfCellsNotOnFaces(faces, ci0, ci1)
         # ifaces = np.arange(nfaces)[innerfaces]
         # A = sparse.coo_matrix((ndofs, ndofs))
         # rows0 = np.repeat(fi0, nloc-1).ravel()
@@ -631,6 +609,10 @@ class CR1(fems.fem.Fem):
     def computeFormTransportSupg(self, du, u, data, method):
         self.computeFormTransportCellWise(du, u, data, type='supg')
         self.computeFormJump(du, u, data.betart)
+    def computeFormTransportLps(self, du, u, data, **kwargs):
+        self.computeFormTransportCellWise(du, u, data, type='centered')
+        self.computeFormJump(du, u, data.betart)
+        self.computeFormLps(du, u, data.betart, **kwargs)
     def massDotSupg(self, b, f, data, coeff=1):
         dim, facesOfCells, dV = self.mesh.dimension, self.mesh.facesOfCells, self.mesh.dV
         # beta, mus, deltas = beta, self.md.mus, self.md.deltas
@@ -714,12 +696,12 @@ class CR1(fems.fem.Fem):
         colors = mesh.bdrylabels.keys()
         bdrydata = self.prepareBoundary(colorsdir=colors)
         A = self.computeMatrixDiffusion(coeff=1)
-        A = self.matrixBoundary(A, bdrydata=bdrydata, method='strong')
+        A = self.matrixBoundaryStrong(A, bdrydata=bdrydata, method='strong')
         b = np.zeros(self.nunknowns())
         rhs = np.vectorize(lambda x,y,z: 1)
         fp1 = self.interpolateCell(rhs)
         self.massDotCell(b, fp1, coeff=1)
-        b = self.vectorBoundaryZero(b, bdrydata)
+        b = self.vectorBoundaryStrongZero(b, bdrydata)
         return self.tonode(splinalg.spsolve(A, b))
 
 
