@@ -13,7 +13,7 @@ from simfempy import fems, tools, meshes
 class P1(fems.fem.Fem):
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
-        self.dirichlet_al = 1
+        self.dirichlet_strong = 1
         self.dirichlet_nitsche = 4
     def setMesh(self, mesh):
         super().setMesh(mesh)
@@ -62,21 +62,19 @@ class P1(fems.fem.Fem):
     def prepareBoundary(self, colorsdir, colorsflux=[]):
         bdrydata = fems.data.BdryData()
         bdrydata.nodesdir={}
-        # bdrydata.nodedirall = np.empty(shape=(0), dtype=np.int)
-        bdrydata.nodedirall = np.empty(shape=(0), dtype=int)
+        bdrydata.nodedirall = np.empty(shape=(0), dtype=self.mesh.faces.dtype)
         for color in colorsdir:
             facesdir = self.mesh.bdrylabels[color]
             bdrydata.nodesdir[color] = np.unique(self.mesh.faces[facesdir].flat[:])
             bdrydata.nodedirall = np.unique(np.union1d(bdrydata.nodedirall, bdrydata.nodesdir[color]))
-        # print(f"{bdrydata.nodedirall=}")
         # bdrydata.nodesinner = np.setdiff1d(np.arange(self.mesh.nnodes, dtype=np.int),bdrydata.nodedirall)
-        bdrydata.nodesinner = np.setdiff1d(np.arange(self.mesh.nnodes, dtype=int),bdrydata.nodedirall)
+        bdrydata.nodesinner = np.setdiff1d(np.arange(self.mesh.nnodes, dtype=self.mesh.faces.dtype),bdrydata.nodedirall)
         bdrydata.nodesdirflux={}
         for color in colorsflux:
             facesdir = self.mesh.bdrylabels[color]
             bdrydata.nodesdirflux[color] = np.unique(self.mesh.faces[facesdir].ravel())
         return bdrydata
-    def matrixBoundary(self, A, bdrydata, method):
+    def matrixBoundaryStrong(self, A, bdrydata, method):
         assert method != 'nitsche'
         nodesdir, nodedirall, nodesinner, nodesdirflux = bdrydata.nodesdir, bdrydata.nodedirall, bdrydata.nodesinner, bdrydata.nodesdirflux
         nnodes = self.mesh.nnodes
@@ -86,7 +84,7 @@ class P1(fems.fem.Fem):
             for i in range(nb): help[i, nodes[i]] = 1
             bdrydata.Asaved[color] = help.dot(A)
         bdrydata.A_inner_dir = A[nodesinner, :][:, nodedirall]
-        help = np.ones((nnodes))
+        help = np.ones((nnodes), dtype=nodedirall.dtype)
         help[nodedirall] = 0
         help = sparse.dia_matrix((help, 0), shape=(nnodes, nnodes))
         # A = help.dot(A.dot(help))
@@ -95,14 +93,14 @@ class P1(fems.fem.Fem):
             diag[nodedirall] = 1.0
             diag = sparse.dia_matrix((diag, 0), shape=(nnodes, nnodes))
         else:
-            bdrydata.A_dir_dir = self.dirichlet_al*A[nodedirall, :][:, nodedirall]
-            diag[nodedirall] = np.sqrt(self.dirichlet_al)
+            bdrydata.A_dir_dir = self.dirichlet_strong*A[nodedirall, :][:, nodedirall]
+            diag[nodedirall] = np.sqrt(self.dirichlet_strong)
             diag = sparse.dia_matrix((diag, 0), shape=(nnodes, nnodes))
             diag = diag.dot(A.dot(diag))
         A = help.dot(A)
         A += diag
         return A
-    def vectorBoundary(self, b, bdrycond, bdrydata, method):
+    def vectorBoundaryStrong(self, b, bdrycond, bdrydata, method):
         assert method != 'nitsche'
         nodesdir, nodedirall, nodesinner, nodesdirflux = bdrydata.nodesdir, bdrydata.nodedirall, bdrydata.nodesinner, bdrydata.nodesdirflux
         x, y, z = self.mesh.points.T
@@ -125,10 +123,10 @@ class P1(fems.fem.Fem):
             # b[nodesinner] -= bdrydata.A_inner_dir * help[nodedirall]
             b[nodedirall] = bdrydata.A_dir_dir * help[nodedirall]
         return b
-    def vectorBoundaryEqual(self, du, u, bdrydata):
+    def vectorBoundaryStrongEqual(self, du, u, bdrydata):
         nodedirall = bdrydata.nodedirall
         du[nodedirall] = u[nodedirall]
-    def vectorBoundaryZero(self, du, bdrydata):
+    def vectorBoundaryStrongZero(self, du, bdrydata):
         du[bdrydata.nodedirall] = 0
     def formBoundary(self, du, u, bdrydata, dirichletmethod):
         assert(dirichletmethod=='new')
@@ -154,7 +152,7 @@ class P1(fems.fem.Fem):
     def computeFormNitscheDiffusion(self, du, u, diffcoff, colorsdir):
         assert u.shape[0]==self.mesh.nnodes
         dim  = self.mesh.dimension
-        massloc = simfempy.tools.barycentric.tensor(d=dim - 1, k=2)
+        massloc = tools.barycentric.tensor(d=dim - 1, k=2)
         massloc = np.diag(np.sum(massloc,axis=1))
         faces = self.mesh.bdryFaces(colorsdir)
         nodes, cells, normalsS = self.mesh.faces[faces], self.mesh.cellsOfFaces[faces,0], self.mesh.normals[faces,:dim]
@@ -169,9 +167,10 @@ class P1(fems.fem.Fem):
         np.add.at(du, simp, -mat)
         mat = np.einsum('f,fk,fjk,fj->f', diffcoff[cells]/dim, normalsS, cellgrads,u[simp]).repeat(dim).reshape(faces.shape[0],dim)
         np.add.at(du, nodes, -mat)
-    def computeMatrixNitscheDiffusion(self, diffcoff, colorsdir, coeff=1):
+    def computeMatrixNitscheDiffusion(self, diffcoff, colors, nitsche_param=None, lumped=True):
+        if nitsche_param==None: nitsche_param=self.dirichlet_nitsche
         nnodes, ncells, dim, nlocal  = self.mesh.nnodes, self.mesh.ncells, self.mesh.dimension, self.nlocal()
-        faces = self.mesh.bdryFaces(colorsdir)
+        faces = self.mesh.bdryFaces(colors)
         cells = self.mesh.cellsOfFaces[faces, 0]
         normalsS = self.mesh.normals[faces, :dim]
         dS = np.linalg.norm(normalsS, axis=1)
@@ -185,19 +184,19 @@ class P1(fems.fem.Fem):
         #     raise ValueError(f"***not found***\n{facenodes=}\n{fnind=} {ind=}")
         cols = np.tile(simp,dim)
         rows = facenodes.repeat(dim+1)
-        mat = np.einsum('f,fk,fjk,i->fij', coeff * diffcoff[cells]/dim, normalsS, cellgrads, np.ones(dim))
+        mat = np.einsum('f,fk,fjk,i->fij', diffcoff[cells]/dim, normalsS, cellgrads, np.ones(dim))
         # mat = np.repeat(mat,dim)
         # print(f"{cols.shape=} {rows.shape=} {mat.shape=}")
         AN = sparse.coo_matrix((mat.ravel(), (rows.ravel(), cols.ravel())), shape=(nnodes, nnodes)).tocsr()
         massloc = tools.barycentric.tensor(d=dim-1, k=2)
         massloc = np.diag(np.sum(massloc,axis=1))
         # print(f"{massloc=}")
-        mat = np.einsum('f,ij->fij', coeff * dS**2/dV*diffcoff[cells], massloc)
+        mat = np.einsum('f,ij->fij', nitsche_param * dS**2/dV*diffcoff[cells], massloc)
         # mat = np.repeat(coeff * diffcoff[cells]/dS, dim)
         rows = np.repeat(facenodes,dim)
         cols = np.tile(facenodes,dim)
         AD = sparse.coo_matrix((mat.ravel(), (rows.ravel(), cols.ravel())), shape=(nnodes, nnodes)).tocsr()
-        return  - AN - AN.T + self.dirichlet_nitsche * AD
+        return  - AN - AN.T + AD
     def computeBdryNormalFluxNitsche(self, u, colors, udir, diffcoff):
         flux= np.zeros(len(colors))
         nnodes, dim  = self.mesh.nnodes, self.mesh.dimension
@@ -215,8 +214,8 @@ class P1(fems.fem.Fem):
             flux[i] = np.einsum('nj,n,ni,nji->', u[simp], diffcoff[cells], normalsS, cellgrads)
             uD = u[nodes]-udir[nodes]
             dV = self.mesh.dV[cells]
-            dS *= self.dirichlet_nitsche * diffcoff[cells] * dS / dV
-            flux[i] -= np.einsum('n,kl,nl->', dS, massloc, uD)
+            flux[i] -= np.einsum('n,kl,nl->', self.dirichlet_nitsche * diffcoff[cells] * dS**2 / dV, massloc, uD)
+            # flux[i] /= np.sum(dS)
         return flux
     # interpolate
     def interpolate(self, f):
@@ -252,7 +251,7 @@ class P1(fems.fem.Fem):
             mass = coeff/(dim+1)*dV.repeat(dim+1)
             rows = self.mesh.simplices.ravel()
             return sparse.coo_matrix((mass, (rows, rows)), shape=(nnodes, nnodes)).tocsr()
-        massloc = barycentric.tensor(d=dim, k=2)
+        massloc = tools.barycentric.tensor(d=dim, k=2)
         mass = np.einsum('n,kl->nkl', coeff*dV, massloc).ravel()
         return sparse.coo_matrix((mass, (self.rows, self.cols)), shape=(nnodes, nnodes)).tocsr()
     def computeBdryMassMatrix(self, colors=None, coeff=1, lumped=False):
@@ -364,7 +363,7 @@ class P1(fems.fem.Fem):
         return self.computeMatrixTransportCellWise(data, type='supg')
     def computeMatrixTransportLps(self, data):
         A = self.computeMatrixTransportCellWise(data, type='centered')
-        A += self.computeMatrixLps(data.beta)
+        A += self.computeMatrixLps(data.betart)
         return A
     def computeMatrixTransportCellWise(self, data, type):
         nnodes, ncells, nfaces, dim = self.mesh.nnodes, self.mesh.ncells, self.mesh.nfaces, self.mesh.dimension
@@ -554,17 +553,31 @@ class P1(fems.fem.Fem):
         up = np.empty(len(colors))
         for i,color in enumerate(colors):
             nodes = self.mesh.verticesoflabel[color]
+            # print(f"{nodes=} {self.mesh.points[nodes]=}")
             up[i] = u[nodes]
         return up
+    def computeLineValues(self, u, colors):
+        raise NotImplementedError()
+        up = np.empty(len(colors))
+        for i,color in enumerate(colors):
+            lines = self.mesh.linesoflabel[color]
+            nodes = np.unique(lines)
+            print(f"{u[nodes]=}")
+            print(f"{self.mesh.points[nodes,0]=}")
+            print(f"{self.mesh.points[nodes,1]=}")
+            import matplotlib.pyplot as plt
+            plt.plot(self.mesh.points[nodes,0], u[nodes])
+            plt.show()
+            # print(f"{np.unique(lines)=}")
+            # print(f"{self.mesh.points[lines]=}")
+        return up
+
     def computeMeanValues(self, u, colors):
         up = np.empty(len(colors))
         for i, color in enumerate(colors):
-            up[i] = self.computeMeanValue(u,color)
+            cells = self.mesh.cellsoflabel[color]
+            up[i] = np.sum(np.mean(u[self.mesh.simplices[cells]],axis=1)*self.mesh.dV[cells])
         return up
-    def computeMeanValue(self, u, color):
-        cells = self.mesh.cellsoflabel[color]
-        # print("umean", np.mean(u[self.mesh.simplices[cells]],axis=1))
-        return np.sum(np.mean(u[self.mesh.simplices[cells]],axis=1)*self.mesh.dV[cells])
 
     #------------------------------
     def test(self):
@@ -572,12 +585,12 @@ class P1(fems.fem.Fem):
         colors = mesh.bdrylabels.keys()
         bdrydata = self.prepareBoundary(colorsdir=colors)
         A = self.computeMatrixDiffusion(coeff=1)
-        A = self.matrixBoundary(A, bdrydata=bdrydata, method='strong')
+        A = self.matrixBoundaryStrong(A, bdrydata=bdrydata, method='strong')
         b = np.zeros(self.nunknowns())
         rhs = np.vectorize(lambda x,y,z: 1)
         fp1 = self.interpolateCell(rhs)
         self.massDotCell(b, fp1, coeff=1)
-        b = self.vectorBoundaryZero(b, bdrydata)
+        b = self.vectorBoundaryStrongZero(b, bdrydata)
         return splinalg.spsolve(A, b)
 
 
