@@ -7,8 +7,9 @@ Created on Sun Dec  4 18:14:29 2016
 import os, sys, importlib
 import meshio
 import numpy as np
+from numpy.lib.shape_base import take_along_axis
 from scipy import sparse
-from simfempy.tools import npext
+from simfempy.tools import npext, timer
 
 #=================================================================#
 class SimplexMesh(object):
@@ -40,19 +41,17 @@ class SimplexMesh(object):
     """
 
     def __repr__(self):
-        s = f"SimplexMesh({self.geometry}): "
-        s += f"dim/nnodes/nfaces/ncells: {self.dimension}/{self.nnodes}/{self.nfaces}/{self.ncells}"
+        s = f"dim/nnodes/nfaces/ncells: {self.dimension}/{self.nnodes}/{self.nfaces}/{self.ncells}"
         s += f"\nbdrylabels={list(self.bdrylabels.keys())}"
         s += f"\ncellsoflabel={list(self.cellsoflabel.keys())}"
         return s
-    def __init__(self, **kwargs):
-        if 'mesh' in kwargs:
-            self.geometry = 'own'
-            mesh = kwargs.pop('mesh')
-        else:
-            raise KeyError("Needs a mesh (no longer geometry)")
+    def __init__(self, mesh, **kwargs):
+        if not isinstance(mesh, meshio.Mesh):
+            raise KeyError(f"Needs a meshio.Mesh, got {type(mesh)}")
+        self.timer = timer.Timer(name="SimplexMesh")
         self._initMeshPyGmsh(mesh)
         self.check()
+        # print(self.timer)
     def check(self):
         if len(np.unique(self.simplices)) != self.nnodes:
             raise ValueError(f"{len(np.unique(self.simplices))=} BUT {self.nnodes=}")
@@ -63,13 +62,43 @@ class SimplexMesh(object):
         faces = np.empty(pos[-1], dtype=np.uint32)
         for i,color in enumerate(colors): faces[pos[i]:pos[i+1]] = self.bdrylabels[color]
         return faces
-
+    def constructInnerFaces(self):
+        self.innerfaces = self.cellsOfFaces[:,1]>=0
+        self.cellsOfInteriorFaces= self.cellsOfFaces[self.innerfaces]
+    def facesOfCellsNotOnInnerFaces(self, ci0, ci1):
+        """
+        assumption: faces are opposite to nodes
+        the order is important, so we need to go through 'faces'
+        """
+        faces = self.faces[self.innerfaces]
+        fi0_bis = np.empty_like(faces)
+        fi1_bis = np.empty_like(faces)
+        # can only serach on by one (and this keeps the order)
+        for i in range(faces.shape[1]):
+            fi0_bis[:,i] = self.facesOfCells[ci0][self.simplices[ci0] == faces[:,i][:,np.newaxis]]
+            fi1_bis[:,i] = self.facesOfCells[ci1][self.simplices[ci1] == faces[:,i][:,np.newaxis]]
+        return fi0_bis, fi1_bis
+        # print(f"{faces=}")
+        # print(f"{self.simplices[ci0]=}")
+        # ind = npext.positionin(faces, self.simplices[ci0])
+        # fi0 =  np.take_along_axis(self.facesOfCells[ci0], ind, axis=1)
+        # ind = npext.positionin(faces, self.simplices[ci1])
+        # fi1 =  np.take_along_axis(self.facesOfCells[ci1], ind, axis=1)
+        # return fi0, fi1
+        # if not np.all(fi0==fi0_bis):
+        #     diff = np.any(fi0 != fi0_bis, axis=1)
+        #     print(f"{fi0[diff]=}\n{fi0_bis[diff]=}")
+        #     print(f"\n{self.simplices[ci0][diff]=}\n{faces[diff]=}")
+        #     assert 0
+        # if not np.all(fi1==fi1_bis):
+        #     diff = np.any(fi1 != fi1_bis, axis=1)
+        #     print(f"{fi1[diff]=}\n{fi1_bis[diff]=}")
+        #     print(f"\n{self.simplices[ci0][diff]=}\n{faces[diff]=}")
+        #     assert 0
     def _initMeshPyGmsh(self, mesh):
-        # only 3d-coordinates
-        assert mesh.points.shape[1] ==3
-        self.points = mesh.points
-        self.nnodes = self.points.shape[0]
+        self.pygmsh = mesh
         self.celltypes = [key for key, cellblock in mesh.cells]
+        assert self.celltypes==list(mesh.cells_dict.keys())
         # for key, cellblock in cells: keys.append(key)
         # print("self.celltypes", self.celltypes)
         if 'tetra' in self.celltypes:
@@ -83,38 +112,48 @@ class SimplexMesh(object):
             self.simplicesname, self.facesname = 'line', 'vertex'
         else:
             raise ValueError(f"something wrong {self.celltypes=} {mesh=}")
-        bdryfacesgmshlist = []
         for key, cellblock in mesh.cells:
             # print(f"{key=} {cellblock=}")
             if key == self.simplicesname: self.simplices = cellblock
-            if key == 'vertex': self.vertices = cellblock
-            if key == self.facesname:
+            elif key == self.facesname:
                 self.facesdata = cellblock
-                # print(f"{key=} {cellblock=}")
-                bdryfacesgmshlist.extend(cellblock)
-        if not hasattr(self,"simplices"):
-            raise ValueError(f"something wrong {self.dimension=}")
+            else:
+                continue
+        if not hasattr(self,"simplices") or not hasattr(self,"facesdata"):
+            raise ValueError(f"something wrong {self=}")
+        assert np.all(self.facesdata==mesh.cells_dict[self.facesname])
+        assert np.all(self.simplices==mesh.cells_dict[self.simplicesname])
+        # only 3d-coordinates
+        assert mesh.points.shape[1] ==3
         # eliminate drangling points
         nnp = len(np.unique(self.simplices))
-        if nnp != self.nnodes:
-            assert np.all(np.unique(self.simplices)==np.arange(nnp))
-            self.points = self.points[:nnp]
-            self.nnodes = nnp
-        # boundaries
-        bdryfacesgmsh = np.array(bdryfacesgmshlist)
+        if not np.all(np.unique(self.simplices)==np.arange(nnp)):
+            msg = f"points in simplices {nnp} but {self.points.shape=}"
+            msg += f"\n{self.celltypes=}\n{mesh.cell_sets=}"
+            msg += f"\n{np.unique(self.simplices)=}"
+            msg += f"\n{mesh.cells_dict=}"
+            raise ValueError(msg)
+        self.points = mesh.points[:nnp]
+        self.nnodes = self.points.shape[0]
         self._constructFacesFromSimplices()
+        self.timer.add("_constructFacesFromSimplices")
         assert self.dimension+1 == self.simplices.shape[1]
         self.ncells = self.simplices.shape[0]
         self.pointsc = self.points[self.simplices].mean(axis=1)
         self.pointsf = self.points[self.faces].mean(axis=1)
         self._constructNormalsAndAreas()
-        self.cell_sets = mesh.cell_sets
-        self._initMeshPyGmsh7(mesh.cells, mesh.cell_sets, bdryfacesgmsh)
+        self.timer.add("_constructNormalsAndAreas")
+        # self.cell_sets = mesh.cell_sets
+        # print(f"{mesh.cell_sets_dict=}")
+        bdrylabelsgmsh = self._initMeshPyGmsh7(mesh.cell_sets, mesh.cells_dict)
+        self.timer.add("_initMeshPyGmsh7")
+        # boundaries
+        # self._constructBoundaryFaces7(bdryfacesgmshlist, bdrylabelsgmsh)
+        self._constructBoundaryFaces7(self.facesdata, bdrylabelsgmsh)
+        self.timer.add("_constructBoundaryFaces7")
+        # print(f"{self.bdrylabels.keys()=}")
         #TODO : remplacer -1 par nan dans les indices
-    def constructInnerFaces(self):
-        self.innerfaces = self.cellsOfFaces[:,1]>=0
-        self.cellsOfInteriorFaces= self.cellsOfFaces[self.innerfaces]
-    def _initMeshPyGmsh7(self, cells, cell_sets, bdryfacesgmsh):
+    def _initMeshPyGmsh7(self, cell_sets, cells_dict):
         # cell_sets: dict label --> list of None or np.array for each cell_type
         # the indices of the np.array are not the cellids !
         # ???
@@ -137,6 +176,7 @@ class SimplexMesh(object):
                     sizes[celltype] += info.shape[0]
                     typesoflabel[ilabel] = celltype
                     ctorderd.append(celltype)
+        # print(f"{self.celltypes=}\n{cellsoflabel=}")
         #correcting the numbering in cell_sets
         n = 0
         for ct in list(dict.fromkeys(ctorderd)):
@@ -144,101 +184,112 @@ class SimplexMesh(object):
             for l, cb in cellsoflabel[ct].items(): cb -= n
             n += sizes[ct]
         self.cellsoflabel = cellsoflabel[self.simplicesname]
-        self.verticesoflabel = {}
-        # print(f"{cellsoflabel=}\n{cellsoflabel.keys()}")
-        # if self.dimension > 1: self.verticesoflabel = cellsoflabel['vertex']
-        # print(f"{self.verticesoflabel=}")
-        # bdry faces
-        # for key, cellblock in cells:
-        #     if key == self.facesnames[self.dimension - 1]: bdryfacesgmsh = cellblock
-        bdrylabelsgmsh = cellsoflabel[self.facesname]
-        self._constructBoundaryFaces7(bdryfacesgmsh, bdrylabelsgmsh)
+        self.verticesoflabel = {k:cells_dict['vertex'][v] for k,v in cellsoflabel['vertex'].items()}
+        self.linesoflabel = {k:cells_dict['line'][v] for k,v in cellsoflabel['line'].items()}
+        if self.facesname not in cellsoflabel:
+            raise ValueError(f"{self.facesname=} not in {cellsoflabel=}")
+        return cellsoflabel[self.facesname]
     def _constructFacesFromSimplices(self):
         simplices = self.simplices
         ncells = simplices.shape[0]
         nnpc = simplices.shape[1]
-        allfaces = np.empty(shape=(nnpc*ncells,nnpc-1), dtype=int)
-        for i in range(ncells):
-            for ii in range(nnpc):
-                mask = np.array( [jj !=ii for jj in range(nnpc)] )
-                allfaces[i*nnpc+ii] = np.sort(simplices[i,mask])
-        s = "{0}" + (nnpc-2)*", {0}"
-        s = s.format(allfaces.dtype)
-        order = ["f0"]+["f{:1d}".format(i) for i in range(1,nnpc-1)]
+        nd = np.logical_not(np.eye(nnpc,dtype=bool)).ravel()
+        allfaces = np.sort(np.tile(simplices, nnpc)[:,nd].reshape(ncells, nnpc, nnpc-1), axis=2).reshape(nnpc*ncells,nnpc-1)
+        s = (nnpc-1)*"{0},"
+        s = s[:-1].format(allfaces.dtype)
+        # order = ["f0"]+["f{:1d}".format(i) for i in range(1,nnpc-1)]
+        order = ["f{:1d}".format(i) for i in range(nnpc-1)]
+        # print(f"{s=} {order=}")
         if self.dimension==1:
             perm = np.argsort(allfaces, axis=0).ravel()
         else:
             perm = np.argsort(allfaces.view(s), order=order, axis=0).ravel()
-        allfacescorted = allfaces[perm]
-        self.faces, indices = np.unique(allfacescorted, return_inverse=True, axis=0)
-        locindex = np.tile(np.arange(0,nnpc), ncells)
-        cellindex = np.repeat(np.arange(0,ncells), nnpc)
+        # print(f"{allfaces=}")
+        # print(f"{perm=}")
+        allfacesorted = allfaces[perm]
+        # print(f"{allfacesorted=}")
+        self.faces, indices = np.unique(allfacesorted, return_inverse=True, axis=0)
+        # print(f"{self.faces=}")
         self.nfaces = self.faces.shape[0]
-        self.cellsOfFaces = -1 * np.ones(shape=(self.nfaces, 2), dtype=int)
         self.facesOfCells = np.zeros(shape=(ncells, nnpc), dtype=int)
-        for ii in range(indices.shape[0]):
-            f = indices[ii]
-            loc = locindex[perm[ii]]
-            cell = cellindex[perm[ii]]
-            self.facesOfCells[cell, loc] = f
-            if self.cellsOfFaces[f,0] == -1: self.cellsOfFaces[f,0] = cell
-            else: self.cellsOfFaces[f,1] = cell
+        locindex = np.tile(np.arange(0,nnpc), ncells).ravel()
+        cellindex = np.repeat(np.arange(0,ncells), nnpc)
+        self.facesOfCells[cellindex[perm],locindex[perm]] = indices
+        unique, indices = np.unique(self.facesOfCells,return_index=True)
+        assert np.all(unique == np.arange(self.nfaces))
+        i0, i1 = np.unravel_index(indices, shape=self.facesOfCells.shape)
+        foc = self.facesOfCells.copy()
+        foc[i0,i1] = -1   
+        unique, indices = np.unique(foc,return_index=True)
+        i2, i3 = np.unravel_index(indices[1:], shape=foc.shape)
+        i = -1*np.ones(self.nfaces, dtype=self.facesOfCells.dtype)
+        i[unique[1:]] = i2
+        self.cellsOfFaces =np.vstack([i0,i]).T
 
     def _constructBoundaryFaces7(self, bdryfacesgmsh, bdrylabelsgmsh):
+        # t = timer.Timer("_constructBoundaryFaces7")
         # bdries
         # bdryfacesgmsh may contains interior edges for len(celllabels)>1
+        # sort along last axis
         bdryfacesgmsh = np.sort(bdryfacesgmsh)
         bdryids = np.flatnonzero(self.cellsOfFaces[:,1] == -1)
-        # print(f"{bdryids=}")
-        # assert np.all(bdryids == np.flatnonzero(np.any(self.cellsOfFaces == -1, axis=1)))
         bdryfaces = np.sort(self.faces[bdryids],axis=1)
-        # print(f"{bdryfacesgmsh=}\n{bdryfaces=}")
-        # ind = np.isin(bdryfacesgmsh, bdryfaces)
-        # print(f"{ind=} {bdryfacesgmsh[ind]=}")
-        # print(f"{bdryfaces=}")
-        nbdryfaces = len(bdryids)
         nnpc = self.simplices.shape[1]
         s = "{0}" + (nnpc-2)*", {0}"
         dtb = s.format(bdryfacesgmsh.dtype)
         dtf = s.format(bdryfaces.dtype)
         order = ["f0"]+["f{:1d}".format(i) for i in range(1,nnpc-1)]
+        # t.add("a")
         if self.dimension==1:
             bp = np.argsort(bdryfacesgmsh.view(dtb), axis=0).ravel()
             fp = np.argsort(bdryfaces.view(dtf), axis=0).ravel()
         else:
             bp = np.argsort(bdryfacesgmsh.view(dtb), order=order, axis=0).ravel()
             fp = np.argsort(bdryfaces.view(dtf), order=order, axis=0).ravel()
+        assert np.all(bdryfaces[fp]==bdryfacesgmsh[bp])
+        bpi = np.argsort(bp)
+        self.bdrylabels = {col:bdryids[fp[bpi[cb]]] for col, cb in bdrylabelsgmsh.items()}
         # print(f"{bp=}")
         # print(f"{fp=}")
 #https://stackoverflow.com/questions/51352527/check-for-identical-rows-in-different-numpy-arrays
-        indices = (bdryfacesgmsh[bp, None] == bdryfaces[fp]).all(-1).any(-1)
-        if not np.all(bdryfaces[fp]==bdryfacesgmsh[bp[indices]]):
-            raise ValueError(f"{bdryfaces.T=}\n{bdryfacesgmsh.T=}\n{indices=}\n{bdryfaces[fp].T=}\n{bdryfacesgmsh[bp[indices]].T=}")
-        bp2 = bp[indices]
-        for i in range(len(fp)):
-            if not np.all(bdryfacesgmsh[bp2[i]] == bdryfaces[fp[i]]):
-                raise ValueError(f"{i=} {bdryfacesgmsh[bp2[i]]=} {bdryfaces[fp[i]]=}")
-        bpi = np.argsort(bp)
-        # bp2i = {bp2[i]:i for i in range(len(bp2))}
-        # print(f"{bp=} \n{bp2=} \n{bpi=} \n{bp2i=} \n{indices=}")
-        binv = -1*np.ones_like(bp)
-        binv[bp2] = np.arange(len(bp2))
-        self.bdrylabels = {}
-        for col, cb in bdrylabelsgmsh.items():
-            # if cb[0] in bp2i.keys():
-            if indices[bpi[cb[0]]]:
-                # for i in range(len(cb)):
-                #     if not bp2i[cb[i]] == binv[cb[i]]:
-                #         raise ValueError(f"{bp2i[cb[i]]} {binv[cb[i]]}")
-                # print(f"{col=}")
-                self.bdrylabels[int(col)] = np.empty_like(cb)
-                # for i in range(len(cb)): self.bdrylabels[int(col)][i] = bdryids[fp[bp2i[cb[i]]]]
-                for i in range(len(cb)): self.bdrylabels[int(col)][i] = bdryids[fp[binv[cb[i]]]]
-            # else:
-            #     assert not indices[bpi[cb[0]]]
+        # t.add("b")
+        ################
+        # print(f"{bdryfacesgmsh.shape=}\n{bdryfaces.shape=}")
+        # print(f"{bp.shape=}\n{fp.shape=}")
+        # print(f"{bdryfacesgmsh[bp]=}\n{bdryfaces[fp]=}")
+        # indices = (bdryfacesgmsh[bp, None] == bdryfaces[fp]).all(-1).any(-1)
+        ################
+        # t.add("c")
+        # print(f"{indices=}")
+        # assert np.all(indices == np.arange(bp.shape[0]))
+
+        # if not np.all(bdryfaces[fp]==bdryfacesgmsh[bp[indices]]):
+        #     raise ValueError(f"{bdryfaces.T=}\n{bdryfacesgmsh.T=}\n{indices=}\n{bdryfaces[fp].T=}\n{bdryfacesgmsh[bp[indices]].T=}")
+        # t.add("d")
+        # bp2 = bp
+        # bp2 = bp[indices]
+        # for i in range(len(fp)):
+        #     if not np.all(bdryfacesgmsh[bp2[i]] == bdryfaces[fp[i]]):
+        #         raise ValueError(f"{i=} {bdryfacesgmsh[bp2[i]]=} {bdryfaces[fp[i]]=}")
+        # t.add("e")
+        # bpi = np.argsort(bp)
+        # binv = -1*np.ones_like(bp)
+        # binv = np.empty_like(bp)
+        # binv[bp2] = np.arange(len(bp2))
+        # self.bdrylabels = {col:bdryids[fp[binv[cb]]] for col, cb in bdrylabelsgmsh.items()}
+        # # t.add("f")
+        # for col, cb in bdrylabelsgmsh.items():
+        #     self.bdrylabels[int(col)] = bdryids[fp[binv[cb]]]
+        #     # if indices[bpi[cb[0]]]:
+        #     #     self.bdrylabels[int(col)] = bdryids[fp[binv[cb]]]
+        #     # else:
+        #     #     assert not indices[bpi[cb[0]]]
+        # t.add("g")
+        # print(t)
     def _constructNormalsAndAreas(self):
+        # t = timer.Timer("_constructNormalsAndAreas")
         elem = self.simplices
-        self.sigma = np.array([2 * (self.cellsOfFaces[self.facesOfCells[ic, :], 0] == ic)-1 for ic in range(self.ncells)])
+        #TODO improve computation of sigma
         if self.dimension==1:
             x = self.points[:,0]
             self.normals = np.stack((np.ones(self.nfaces), np.zeros(self.nfaces), np.zeros(self.nfaces)), axis=-1)
@@ -276,32 +327,53 @@ class SimplexMesh(object):
             dz2 = z[elem[:, 2]] - z[elem[:, 0]]
             dz3 = z[elem[:, 3]] - z[elem[:, 0]]
             self.dV = (1/6) * np.abs(dx1*(dy2*dz3-dy3*dz2) - dx2*(dy1*dz3-dy3*dz1) + dx3*(dy1*dz2-dy2*dz1))
-        for i in range(self.nfaces):
-            i0, i1 = self.cellsOfFaces[i, 0], self.cellsOfFaces[i, 1]
-            if i1 == -1:
-                xt = np.mean(self.points[self.faces[i]], axis=0) - np.mean(self.points[self.simplices[i0]], axis=0)
-                if np.dot(self.normals[i], xt)<0:  self.normals[i] *= -1
-            else:
-                xt = np.mean(self.points[self.simplices[i1]], axis=0) - np.mean(self.points[self.simplices[i0]], axis=0)
-                if np.dot(self.normals[i], xt) < 0:  self.normals[i] *= -1
-        # self.sigma = np.array([1.0 - 2.0 * (self.cellsOfFaces[self.facesOfCells[ic, :], 0] == ic) for ic in range(self.ncells)])
+        ind = np.arange(self.ncells)
+        self.sigma = 2* np.equal(self.cellsOfFaces[self.facesOfCells[ind, :], 0], ind[:,np.newaxis]) -1
+        ib = np.arange(self.nfaces)[self.cellsOfFaces[:, 1] == -1]
+        xt = np.mean(self.points[self.faces[ib]], axis=1) - np.mean(self.points[self.simplices[self.cellsOfFaces[ib, 0]]], axis=1)
+        m = np.einsum('nk,nk->n', self.normals[ib], xt)<0
+        self.normals[ib[m]] *= -1
+        ib = np.arange(self.nfaces)[self.cellsOfFaces[:, 1] != -1]
+        xt = np.mean(self.points[self.simplices[self.cellsOfFaces[ib, 1]]], axis=1) - np.mean(self.points[self.simplices[self.cellsOfFaces[ib, 0]]], axis=1)
+        m = np.einsum('nk,nk->n', self.normals[ib], xt)<0
+        self.normals[ib[m]] *= -1
     # ----------------------------------------------------------------#
-    def write(self, filename, dirname = None, point_data=None):
+    def _getfilename(self, filename, dirname=None):
         if dirname is not None:
             dirname = dirname + os.sep + "mesh"
-            if not os.path.isdir(dirname) :
+            if not os.path.isdir(dirname):
                 os.makedirs(dirname)
             filename = os.path.join(dirname, filename)
+        return filename
+    # ----------------------------------------------------------------#
+    def writemeshio(self, filename, dirname = None, data=None):
+        assert filename.split('.')[-1] == 'msh'
+        filename = self._getfilename(filename, dirname)
+        self.pygmsh.write(filename, file_format="gmsh22")
+    # ----------------------------------------------------------------#
+    def write(self, filename, dirname = None, data=None):
+        filename = self._getfilename(filename, dirname)
         if self.dimension == 1:
             cells = {'lines': self.simplices}
             cells['vertex'] = self.facesdata
         elif self.dimension ==2:
-            cells = {'triangle': self.simplices}
-            cells['line'] = self.facesdata
+            # cells = {'triangle': self.simplices}
+            # cells['line'] = self.facesdata
+            cells = [('triangle', self.simplices), ('line', self.facesdata)]
+            # cells = [('triangle', self.simplices)]
         else:
-            cells = {'tetra': self.simplices}
-            cells['triangle'] = self.facesdata
-        mesh = meshio.Mesh(self.points, cells)
+            cells = [('tetra', self.simplices)]
+            # cells = [('tetra', self.simplices), ('triangle', self.facesdata)]
+        # meshio.write_points_cells(filename, self.points, cells, file_format="gmsh")
+        args = {'points': self.points, 'cells':cells}
+        if data is not None:
+            if 'point' in data:
+                args['point_data'] = data['point']
+            if 'cell' in data:
+                # print(f"{data['cell']=}")
+                args['cell_data'] = {k: [data['cell'][k]] for k in data['cell'].keys()}
+        mesh = meshio.Mesh(**args)
+        # print(f"{mesh=}")
         meshio.write(filename, mesh)
     # ----------------------------------------------------------------#
     def computeSimpOfVert(self, test=False):
