@@ -24,7 +24,7 @@ def backtracking(f, x0, dx, resfirst, sdata, verbose=False):
     it = 0
     if verbose:
         print("{} {:>3} {:^10} {:^10}  {:^9}".format("bt", "it", "resnorm", "resfirst", "step"))
-        print("{} {:3} {:10.3e} {:10.3e}  {:9.2e}".format("bt", it, resnorm, resfirst, step))
+        print(f"bt {it:3} {resnorm:10.3e} {resfirst:10.3e} {step:9.2e}")
     while resnorm > (1-c*step)*resfirst and it<maxiter:
         it += 1
         step *= omega
@@ -32,7 +32,7 @@ def backtracking(f, x0, dx, resfirst, sdata, verbose=False):
         res = f(x)
         resnorm = np.linalg.norm(res)
         if verbose:
-            print("{} {:3} {:10.3e}  {:9.2e}".format("bt", it, resnorm, step))
+            print(f"bt {it:3} {resnorm:10.3e} {resfirst:10.3e} {step:9.2e}")
     return x, res, resnorm, step
 
 #----------------------------------------------------------------------
@@ -41,6 +41,7 @@ class Baseopt:
         self.f, self.sdata, self.verbose = f, sdata, verbose
         if not hasattr(sdata, 'nbase'): raise ValueError(f"please give 'nbase' in sdata")
         self.nbase, self.nused = sdata.nbase, 0
+        self.sdata = sdata
         self.du = np.zeros(shape=(self.nbase,n))
         self.u0, self.u, self.r = np.zeros(n), np.zeros(n), np.zeros(n)
         self.ind = []
@@ -48,11 +49,14 @@ class Baseopt:
     def res(self, x):
         self.u[:] = self.u0[:]
         for i in range(self.nused):
+            # print(f"{i=} {np.linalg.norm(self.du[self.ind[i]])=}")
             self.u += x[i]*self.du[self.ind[i]]
-        # print(f"{x=} {np.linalg.norm(self.u0)=} {np.linalg.norm(self.u)=}")
         self.r = self.f(self.u)
+        # print(f"{x=} {np.linalg.norm(self.u0)=}  {np.linalg.norm(self.u)=} {np.linalg.norm(self.r)=}")
+        # x0 = x*x*(1-x)*(1-x)
         return self.r.dot(self.r)
     def step(self, u0, du, resfirst):
+        # print(f"+++++++ {np.linalg.norm(u0)=} {np.linalg.norm(du)=} {resfirst=}")
         from scipy import optimize
         self.u0[:] = u0[:]
         self.resfirst = resfirst
@@ -66,17 +70,28 @@ class Baseopt:
         else:
             self.nused += 1
         self.ind.append(self.last)
-        # print(f"{self.iter} {self.ind=}")
+        # print(f"{self.iter} {self.ind=} {np.linalg.norm(self.u0)}")
         self.du[self.last] = du
         x0 = np.zeros(self.nused)
+        self.res(x0)
+        # x0[-1] = 1
+        self.iter += 1
         x0[-1] = 1
+        # print(f"????????????? {self.iter=}")
+        # if self.iter==1:
+        #     resnorm = self.res(x0)
+        #     return self.u, self.r, resnorm, 1
+        # method = 'COBYLA'
+        # method = 'TNC'
         method = 'BFGS'
         # method = 'CG'
-        out = optimize.minimize(fun=self.res, x0=x0, method=method, options={'disp':False, 'maxiter':10, 'gtol':1e-2})
-        # print(f"{out=}")
-        # print(f"{self.resfirst=} {np.linalg.norm(self.r)=}")
-        self.iter += 1
-        return self.u, self.r, out.fun, out.x
+        options={'disp':False, 'maxiter':self.sdata.maxter_stepsize, 'gtol':1e-4}
+        out = optimize.minimize(fun=self.res, x0=x0, method=method, options=options)
+        if np.linalg.norm(self.r) > self.resfirst:
+            print(f"{np.linalg.norm(self.r)=} {self.resfirst=}")
+            print(f"{out=}")
+            assert 0
+        return self.u, self.r, np.linalg.norm(self.r), out.x
 
 #--------------------------------------------------------------------
 def newton(x0, f, computedx=None, sdata=None, verbose=False, jac=None, maxiter=None, resred=0.1):
@@ -98,11 +113,14 @@ def newton(x0, f, computedx=None, sdata=None, verbose=False, jac=None, maxiter=N
     xnorm = np.linalg.norm(x)
     res = f(x)
     resnorm = np.linalg.norm(res)
+    # print(f"--------- {np.linalg.norm(x)=} {resnorm=}")
     tol = max(atol, rtol*resnorm)
     toldx = max(atoldx, rtoldx*xnorm)
+    name = 'newton'
+    if hasattr(sdata,'addname'): name += '_' + sdata.addname
     if verbose:
-        print("{} {:>3} {:^10} {:^10} {:^10} {:^5} {:^5} {:^3} {:^9}".format("newton", "it", "|x|", "|dx|", '|r|','rhodx','rhor','lin', 'step'))
-        print("{} {:3} {:10.3e} {:^10} {:10.3e} {:^9} {:^5} {:^5} {:^3}".format("newton", 0, xnorm, 3*'-', resnorm, 3*'-', 3*'-', 3*'-', 3*'-'))
+        print("{} {:>3} {:^10} {:^10} {:^10} {:^5} {:^5} {:^3} {:^9}".format(name, "it", "|x|", "|dx|", '|r|','rhodx','rhor','lin', 'step'))
+        print("{} {:3} {:10.3e} {:^10} {:10.3e} {:^9} {:^5} {:^5} {:^3}".format(name, 0, xnorm, 3*'-', resnorm, 3*'-', 3*'-', 3*'-', 3*'-'))
     # while( (resnorm>tol or dxnorm>toldx) and it < maxiter):
     dx, step, resold = None, None, np.zeros_like(res)
     iterdata = newtondata.IterationData(resnorm)
@@ -114,18 +132,22 @@ def newton(x0, f, computedx=None, sdata=None, verbose=False, jac=None, maxiter=N
             dx, liniter = linalg.solve(J, -res), 1
         else:
             dx, liniter = computedx(-res, x, iterdata)
+        assert dx.shape == x0.shape
         resold[:] = res[:]
+        # print(f"--------- {np.linalg.norm(x)=} {resnorm=}")
         if sdata.steptype == 'rb':
             x, res, resnorm, step = bt.step(x, dx, resnorm)
         else:
-            x, res, resnorm, step = backtracking(f, x, dx, resnorm, sdata)
+            x, res, resnorm, step = backtracking(f, x, dx, resnorm, sdata, verbose=True)
+        res2 = f(x)
+        assert np.allclose(res, res2)
         iterdata.newstep(dx, liniter, resnorm, step)
         xnorm = linalg.norm(x)
         if verbose:
-            print(f"newton {iterdata.iter:3} {xnorm:10.3e} {iterdata.dxnorm[-1]:10.3e} {resnorm:10.3e} {iterdata.rhodx:5.2f} {iterdata.rhor:5.2f} {liniter:3d} {step}")
+            print(f"{name} {iterdata.iter:3} {xnorm:10.3e} {iterdata.dxnorm[-1]:10.3e} {resnorm:10.3e} {iterdata.rhodx:5.2f} {iterdata.rhor:5.2f} {liniter:3d} {step}")
         if xnorm >= divx:
-            return (x, maxiter)
-    return (x,iterdata.iter)
+            return (x, maxiter, np.mean(iterdata.liniter))
+    return (x,iterdata.iter, np.mean(iterdata.liniter))
 
 
 # ------------------------------------------------------ #
