@@ -6,19 +6,20 @@ sys.path.insert(0,simfempypath)
 import simfempy.meshes.testmeshes as testmeshes
 from simfempy.applications.beam import Beam
 import simfempy.applications.problemdata
-from simfempy.test.test_analytic import test_analytic
+from simfempy.tools.comparemethods import CompareMethods
 
 #----------------------------------------------------------------#
 def test(**kwargs):
     data = simfempy.applications.problemdata.ProblemData()
     exactsolution = kwargs.pop('exactsolution', 'Linear')
-    paramargs = {}
+    paramsdict = {}
     data.params.scal_glob['EI'] = kwargs.pop('EI', 1)
     createMesh = testmeshes.unitline
     data.bdrycond.set("Clamped", [10000,10001])
     linearsolver = kwargs.pop('linearsolver', 'umf')
     applicationargs= {'problemdata': data, 'exactsolution': exactsolution, 'linearsolver': linearsolver}
-    return test_analytic(application=Beam, createMesh=createMesh, paramargs=paramargs, applicationargs=applicationargs, **kwargs)
+    comp =  CompareMethods(createMesh=createMesh, paramsdict=paramsdict, application=Beam, applicationargs=applicationargs, **kwargs)
+    return comp.compare()
 
 #================================================================#
 if __name__ == '__main__':
